/*---------------------------------------------------------------------------
File:					Pate.java
Package:				PateLib  Version 1.0
Author:					Anna O. Bilska (Anya) V1.0 08/07/96
--------------------------------------------------------------------------*/

/* 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme
 * Computer Science Department
 * Duke University
 * August 1996
 * Supported by National Science Foundation DUE-9596002 and DUE-9555084.
 *
 * Copyright (c) 1996
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

package PateLib;

import java.util.*;
import java.io.*;
import java.awt.*;

// a class which saves the derivation
// to a file


// *****************************************************
// ************METHOD INDEX*****************************
// *****************************************************
// CONSTRUCTORS:
// public DerivationSaver(Vector d, Vector p, Grammar g)
// METHODS:
// public void write(String filename) throws Exception


public class DerivationSaver{

    public DerivationSaver(Vector d, Vector p, Grammar g)
    // constructor
    // d: the derivation
    // p: productions used and how
    // g: grammar 
    {
       myWords = d;
       myProdsUsed = p;
       myGram = g;
       len = myWords.size();
       shiftee = new Shifter();
    }

   
    public void write(String filename) throws Exception
    // write the derivation to a file
    {
       FileOutputStream out = new FileOutputStream(filename);
       PrintWriter ps = new PrintWriter(out);
       int temp;
       int k;

       //output the grammar
       ps.println("Grammar: ");
       for(myGram.FirstProd(); !myGram.ProdsDone(); myGram.NextProd())
       {
          ps.println(myGram.CurrentLeftProd() + "->" + 
             myGram.CurrentRightProd());
       }

       // output the derivation
       ps.println(" ");
       ps.print("Derivation for ");
       ps.println((String)myWords.firstElement() + ":");
       ps.print("S=>");
       temp = ((Integer)myProdsUsed.elementAt(len-2)).intValue();
       k = shiftee.GetProd(temp);
       ps.print("   (");
       ps.print(myGram.LeftProdAt(k));
       ps.print("->");
       ps.print(myGram.RightProdAt(k));
       ps.println(")");
       
       for (int i=(len-2); i>0; i--)
       {
          ps.print(" =>");
          ps.print((String)myWords.elementAt(i));
          temp = ((Integer)myProdsUsed.elementAt(i-1)).intValue();
          k = shiftee.GetProd(temp);
          ps.print("   (");
          ps.print(myGram.LeftProdAt(k));
          ps.print("->");
          ps.print(myGram.RightProdAt(k));
          ps.println(")");
       }
       
       // output the last word 
       if (!myWords.isEmpty())
          ps.print(" =>" + (String)myWords.firstElement());

       ps.close();
    }

    private Shifter shiftee;
    private Grammar myGram;
    private Vector myWords;
    private Vector myProdsUsed;
    private int len;
}



