package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
---------------------------------------------------------------------------
File	    :		GraphFrame.java
Package	    :		PateLib  Version 1.0
Author	    :		Jason Robert Salemme
Programmed  :		8 Aug 96	
Modified    :           Ted Hung 8/99
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme, Ted Hung
 * Computer Science Department
 * Duke University
 * August 1999
 * Supported by National Science Foundation DUE-9596002, DUE-9555084,
 *  and DUE-9752583.
 *
 * Copyright (c) 1999
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
/**  
 * This class is a frame that will allow the user to visualize a set of
 * productions for a grammar.  It requires three parameters when
 * called.  The first is the grammar itself.  The second is a boolean to
 * tell the program whether or not to "visualize" those variables which
 * have a terminating production or derives one.  The third is a vector that
 * contains the variables that <b>DO NOT</b> have or derive terminating
 * productions.  If the second parameter is FALSE then just send a blank 
 * vector.
 * 
 * @author Jason R. Salemme
 * @version 1.0, 08 Aug 96
*/
public class GraphFrame extends Frame implements ActionListener, MouseListener, MouseMotionListener
{

/**
 * Allocates a new instance of the class GraphFrame.  This is just like a Frame
 * so use the resize() and show() method to display it.
 * @param g the grammar to visualize
 * @param showTerms indicates if termination circles should be shown
 * @param nonTermVars Vector of variable that do not terminate
 */
    public GraphFrame()
    {
	super();
	setFont(Params.normalFont);
	Panel p = new Panel();
	p.setLayout(new FlowLayout());
	Button b;
	p.add(b = new Button("Help"));
	b.addActionListener(this);
	p.add(b = new Button("Show"));
	b.addActionListener(this);
	p.add(b = new Button("Close"));
	b.addActionListener(this);
	setLayout(new BorderLayout());
	add("South", p);

	this.addMouseMotionListener(this);
	this.addMouseListener(this);

	addWindowListener(
	    new WindowAdapter()
	    {
		public void windowClosing(WindowEvent e)
		{
		    GraphFrame.this.dispose();
		}
	    }
	);
    }

    public void setParameters(Grammar g, boolean showTerms, Vector nonTermVars)
    {
       this.graph = g;
       this.showTerms = showTerms;
       this.nonTermVars = nonTermVars;
       
       //reset to the default values
       states = new Vector();
       nStates = 0;
       transitions = new Vector();
       nTransitions = 0;
       current = -1;
       selected = -1;
       loopW = 20;
       loopH = 50;
       Tside = 10;
       Theight = 8;
       addLength = 30;
       offscreen = null;
       offgraph = null;
       bottom = 30;
       top = 20;
       termPix = 4;
       init = false;
       wasMoved = false;
       checkTrans = new Hashtable(151);
       totalTrans = 0;
       userTrans = 0;
       checkTerms = new Hashtable(151);
       totalTerms = 0;
       userTerms = 0;
    }

    public void actionPerformed(ActionEvent e)
    {
	String arg;
	if(e.getSource() instanceof Button)
	{
	    arg = ((Button)e.getSource()).getLabel();
	    if(arg.equals("Help"))
	    {
		HelpWindow help = HelpWindowHTML.getHelpWindow("PATE",
		    Params._hd_bkcolor);
		help.setVisible(true);
	    }
	    else if(arg.equals("Help Text"))
	    {
		HelpWindow help = HelpWindowText.getHelpWindow("PATE",
		    Params._hd_bkcolor);
		help.setVisible(true);
	    }
	    else if(arg.equals("Show"))
            {
		//Show the transitions of the graph
                doneVisualizing = true;
		ShowTransitions();
		return;
	    }
	    else if(arg.equals("Close"))
            {
		this.dispose();
	    }
	}
    }

    public void update(Graphics g){
    /* This method overrides the Graphics update method.  It draws the images
     * off of the screen and THEN draws the full image.  This avoids the nasty
     * litttle flicker that would normally be seen.
     */
	Dimension d = getSize();

	if (offscreen == null){
	    offscreen = createImage(d.width, d.height);
	    offgraph = offscreen.getGraphics();
	}
	else if (offscreen.getWidth(this) != d.width || offscreen.getHeight(this) != d.height){
	    offscreen = createImage(d.width, d.height);
	    offgraph = offscreen.getGraphics();
	}
	if (offgraph == null)
	    paint(g);
	else{
	    paint(offgraph);
	    g.drawImage(offscreen, 0, 0, null);
	}
    }

    public void paint(Graphics g)
    //postcond:	"Paints the states and transitions to the frame.
    {
	if(!init)
	{
	    DisplayStates();
	    init = true;
	}
	
	g.setFont(new Font("TimesRoman", Font.PLAIN, 10));
	Dimension d = getSize();

	g.setColor(this.getBackground());
	g.fillRect(0, 0, d.width, d.height);

        drawTransitions(g);
	drawStates(g);
    }

    public void ShowTransitions(){
    //takes the check hashtable and adds the transitions to the states vector
    //of transistions (a little bit weird since transitions are stored by
    //the Points that the state is at)
	Vector v;
	State s1;

	for(int k = 0; k < nStates; k++){
	    s1 = (State)states.elementAt(k);
	    //if the state derives or has terminals than set it accordingly
	    if(checkTerms.get(s1.Name()) != null)
		s1.Terminates(true);
	    v = (Vector)checkTrans.get(s1.Name());
	    if(v.size() > 0){
		s1.setTransitions((Vector)v.clone());
		transitions.addElement(new Integer(k));
		nTransitions++;
	    }
	}
	repaint();
    }

    public void DisplayStates(){
    //postcond: adds the states to the states vector as well as its transitions
	Dimension d = getSize();

	//Add the states
	/*for(graph.FirstProd(); !graph.ProdsDone(); graph.NextProd()){	    
	    //says that the variable is not a terminator because the user
	    //will eventually click on those that are to diplay the terminating
	    //state (this is stored in the transition vectors)
	    add(0, 0, graph.CurrentLeftProd(), false);
	}*/
        for(Enumeration e = graph.getVariables().elements(); e.hasMoreElements(); )
        {
           add(0, 0, (String) e.nextElement(), false);
        }

	//Move the states to form a circle on the screen
	int x, y;
	double	angle;
	int xmid = d.width / 2;
	int ymid = (d.height - bottom - top) / 2 + top;
	int xrad = d.width / 2 - 50;
	int yrad = (d.height - bottom - top) / 2 - 50;

	for(int k = 0; k < nStates; k++){
	    angle = 2 * Math.PI * k / nStates;
	    x = xmid + (int)(Math.cos(angle) * xrad);
	    y = ymid + (int)(Math.sin(angle) * yrad);
	    ((State)states.elementAt(k)).move(x, y);
	}

	//Add the transitions to a check hashtable for checking while the 
	//user input the transitions him/herself
	String right, left, t;
	int index;
	Vector v;
	State s;
	Point p;

	for(graph.FirstProd(); !graph.ProdsDone(); graph.NextProd()){
	    left = graph.CurrentLeftProd();

	    //Add the variable if it has or derives terminals
	    if(checkTerms.get(left) == null && 
	      nonTermVars.indexOf(left) == -1){
		checkTerms.put(left, new Integer(1));
		totalTerms++;
	    }

	    //Put the variable in the hashtable if not there
	    //I am using the variable as the hashtable key
	    if((Vector)checkTrans.get(left) == null){
		v = new Vector();
		checkTrans.put(left, v);
	    }
	    //Add the transitions to the hashtable vectors
	    v = (Vector)checkTrans.get(left);
	    right = graph.CurrentRightProd();
	    
	    //Do this for unit production visualization.  We only
	    //want the unit production transitions.
	    if(!showTerms){
		//Only those unit productions
		if(graph.getEnd(right, 0) == right.length()){
		    index = StatesContain(right);
		    //Add transition if right variable has a prod
		    if(index != -1){
			p = ((State)states.elementAt(StatesContain(right))).getPoint();
			if(v.indexOf(p) == -1){
			    v.addElement(p);
			    totalTrans++;
			}
		    }
		}
	    }
	    //Do this for the useless produciton visualization.  We
	    //want to show everything.
	    else{
		for(int k = 0; k < right.length(); k++){
		    if(Character.isUpperCase(right.charAt(k))){
			t = right.substring(k, graph.getEnd(right, k));
			index = StatesContain(t);
			if(index != -1){
			    s = (State)states.elementAt(StatesContain(t));
			    p = s.getPoint();

	//add the transition to the check only if not there already
	//If you want transitions to itself remove the 
	//!left.equals(t) part.  It should then allow a loop to be
	//draw to itself (the code should already be here.
			    if(!left.equals(t) && v.indexOf(p) == -1){
				v.addElement(p);
				totalTrans++;
			    }
			}
		    }
		}
	    }
	}
	repaint();	//repaint to show the states
    }

    public void drawTransitions(Graphics g){
    //Draws the transition lines
	int index;    

	g.setColor(Color.black);
	for(int i = 0; i < nTransitions; i++){
	    index = ((Integer)transitions.elementAt(i)).intValue();
	    State fromState = (State)states.elementAt(index);
	    Point from = fromState.getPoint();
	    Point to;

	    Vector v = ((State)states.elementAt(index)).getTransitions();
	    for(int k = 0; k < v.size(); k++){
		to = (Point)v.elementAt(k);
		if(from.equals(to)){	//Draw a loop if transition to self
		    g.drawOval(to.x - loopW/2, to.y - loopH, loopW, loopH);
		    drawSelfArrow(from, g);
		}
		else{			//Draw a line to other state
		    g.drawLine(from.x, from.y, to.x, to.y);
		    drawArrow(from, to, g);
		}
	    }
	}
    }

    public void drawStates(Graphics g)
    {
    //Draws the State circles
	State state;
	Point p;
	int half = CIRCLERAD / 2;
	int halft = termPix / 2;
	FontMetrics fm = g.getFontMetrics();

	for(int i = 0; i < nStates; i++){
	    state = (State)states.elementAt(i);
	    p = state.getPoint();
	    g.setColor(Color.yellow);
	    g.fillOval(p.x - half, p.y - half, CIRCLERAD, CIRCLERAD);
	
	    /*if(selected == i){
		g.setColor(Color.red);
		g.fillOval(p.x - half, p.y - half, CIRCLERAD, CIRCLERAD);
	    }*/

	    g.setColor(Color.black);
	    g.drawOval(p.x - half, p.y - half, CIRCLERAD, CIRCLERAD);
	    //If a terminating variable draw a second inner circle
	    if(state.isTerminator() && showTerms)
		g.drawOval(p.x - half + halft, p.y - half + halft,
			   CIRCLERAD - termPix, CIRCLERAD - termPix);

	    //Draw the state label
	    String s = state.Name();
	    int w = fm.stringWidth(s);
	    int h = fm.getAscent();
	    g.drawString(s, p.x - ((int)w/2), p.y + ((int)h/2));
	}
    }

    public void drawArrow(Point from, Point to, Graphics g)
    {
    //Draws the arrow on the transition line pointing to point to

	Point top = FindPoint(from, to, CIRCLERAD/2);
	Point bottom = FindPoint(from, to, (CIRCLERAD/2 + Theight));
	DrawTriangle(top, bottom, g);
    }

    public Point FindPoint(Point a, Point b, int pix){
    //finds the point pix pixles away from b on the line from a to b
	Point p = new Point(0,0);

	int dx = b.x - a.x;
	int dy = b.y - a.y;
	double length = Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));
	double divFactor = ((double)pix)/length;
	int subx = Math.abs((int)(dx * divFactor));
	int suby = Math.abs((int)(dy * divFactor));

	//Special cases (avoid the nasty divide by zero!!)
	if (dx < 0)
	    p.x = b.x + subx;
	else if (dx > 0)
	    p.x = b.x - subx;
	else
	    p.x = b.x;
	if (dy < 0)
	    p.y = b.y + suby;
	else if (dy > 0)
	    p.y = b.y - suby;
	else
	    p.y = b.y;

	return p;

    }

    public void DrawTriangle(Point t, Point b, Graphics g){
    //Draws the triangle on the line
	Point p1 = new Point(0,0);
	Point p2 = new Point(0,0);

	int dx = b.x - t.x;
	int dy = b.y - t.y;	
	int halfSide = Tside/2;

	//If triangle is sideways
	if(dy == 0){
	    p1.x = b.x;
	    p1.y = b.y + halfSide;
	    p2.x = b.x;
	    p2.y = b.y - halfSide;
	}
	//If triangle is upright
	else if(dx == 0){
	    p1.x = b.x + halfSide;
	    p1.y = b.y;
	    p2.x = b.x - halfSide;
	    p2.y = b.y;
	}
	//If somewhere inbetween
	else{
	    //Find point perpend. to (t,b) and is atleast addlength pixles away
	    p1.x = b.x + dy * addLength;
	    p1.y = b.y - dx * addLength;
	    //Find point half of the triangle side away on perp. line
	    p2 = FindPoint(p1, b, halfSide);
	    int dx2 = p2.x - b.x;
	    int dy2 = p2.y - b.y;
	    
	    p1.x = b.x - dx2;
	    p1.y = b.y - dy2;
	}

	drawPoly(t, p1, p2, g);
    }

    public void drawSelfArrow(Point p, Graphics g){
    //Draws an arrow at the top of the loop
	Point p1 = new Point(0,0);
	Point p2 = new Point(0,0);
	Point p3 = new Point(0,0);
	int w = Tside/2;

	p1.x = p.x - w;
	p1.y = p.y - loopH;
	p2.x = p.x + w - 1;
	p2.y = p.y - loopH - w;
	p3.x = p.x + w - 1;
	p3.y = p.y - loopH + w;

	drawPoly(p1, p2, p3, g);
    }

    public void drawPoly(Point p1, Point p2, Point p3, Graphics g){
    //draws a filled in triangle
	Polygon poly = new Polygon();
	poly.addPoint(p1.x, p1.y);
	poly.addPoint(p2.x, p2.y);
	poly.addPoint(p3.x, p3.y);
	g.fillPolygon(poly);    
    }

    public int find(int x, int y){
    //postcond: finds the index of the state located within the radius of
    //		point (x,y).  Returns -1 otherwise
    
	Point p;

	for(int i = 0; i < nStates; i++){
	    p = ((State)states.elementAt(i)).getPoint();
	    if(p.x - CIRCLERAD/2 <= x && x <= p.x + CIRCLERAD/2 &&
	       p.y - CIRCLERAD/2 <= y && y <= p.y + CIRCLERAD/2)
		return i;
	}
	return -1;
    }

    public void add(int x, int y, String name, boolean term){
    //postcond: adds the state to the vector of states.  If the state is a 
    //		repeat it changes the terminator value of it is true

	int index = StatesContain(name);
	if(index == -1){
	    states.addElement(new State(name, term, x, y));
	    nStates++;
	}
	else{
	    if(term && !((State)states.elementAt(index)).isTerminator()){
		states.setElementAt(new State(name, term, x, y), index);
	    }
	}
    }

    public int StatesContain(String name){
    //postcond:	returns the index of the state the has the name name
    //		returns false otherwise
	for(int k = 0; k < nStates; k++){
	    if(name.equals(((State)states.elementAt(k)).Name()))
		return k;
	}
	return -1;
    }

    public void makeTransition(int from, int to){
    //postcond: sets the transition in the from states with the to point

	Point p = ((State)states.elementAt(to)).getPoint();
	if(!((State)states.elementAt(from)).hasTransition(p)){
	    ((State)states.elementAt(from)).addTransition(p);
	    Integer f = new Integer(from);
	    if(transitions.indexOf(f) == -1){
		transitions.addElement(f);
		nTransitions++;
	    }
	}
    }

    public void addDoubleCircle(int current){
    //postcond:	displays a double circle if the variable haas or derives a
    //		terminating production

	if(userTerms < totalTerms){
	    State s = ((State)states.elementAt(current));
	    if(checkTerms.get(s.Name()) != null){
		s.Terminates(true);
		userTerms++;
		if(userTerms == totalTerms)
                {
		    repaint();
		    OKDialog dg = new OKDialog("You have added the last\ntermination circle for the\noriginal set of productions.", this, "Congratulations!!!");
		    dg.showResize();
		}
		    
	    }
	    else if(nonTermVars.indexOf(s.Name()) == -1){
		OKDialog dg = new OKDialog("You have already made\n"+s.Name()+" a variable that\nhas or derives a terminating\nproduction.", this, "Oooops!!!");
		dg.showResize();
	    }
	    else{
		OKDialog dg = new OKDialog("The variable "+s.Name()+"\ndoes not have or derive\n a production that terminates.", this, "No, no, NO!!!");
		dg.showResize();
	    }
	}
	else{
	    OKDialog dg = new OKDialog("You have already diplayed\nall variables that have\nor derive terminating productions!", this, "Hello!!???");
	    dg.showResize();
	}
        repaint();
    }

    public void Select(int current)
    {
    //Highlights the current state.  If one is already highlighted
    //It draws a line between the two

	if (selected == -1){
	    selected = current;
	}
        else{
	    if(userTrans < totalTrans){
		if(IsValidTrans(selected, current)){
		    if(!TransAlreadyMade(selected, current)){
			makeTransition(selected, current);
			selected = -1;
			userTrans++;
			if(userTrans == totalTrans){
			    repaint();
			    OKDialog dg = new OKDialog("You added the last \ntransition for the\noriginal set of productions!!!", this, "Congratulations!!!");
                            doneVisualizing = true;
			    dg.showResize();
			}
		    }
		    else{
			OKDialog dg = new OKDialog("You have already added\nthat transition!!!", this, "Uhhhhhh!!!");
			dg.showResize();
		    }
		}
		else{
		    String from = ((State)states.elementAt(selected)).Name();
		    String to = ((State)states.elementAt(current)).Name();
		    OKDialog dg = new OKDialog("The transition:\n" + from +
			" -> " + to + "\n does not exist in the\noriginal set of productions", this, "Sorry!!!");
		    dg.showResize();
		    selected = -1;
		}
	    }
	    else{
		OKDialog dg = new OKDialog("You have already displayed\nall the transitions for\nthe set of productions!", this, "You are already done!!!");
		doneVisualizing = true;
                dg.showResize();
	    }
	}
    }

    public boolean IsValidTrans(int from, int to){
    //postcond:	returns true if it is a valid transition in the check hashtable
    //		and if it has not alreay been added to the states.
	String fname = ((State)states.elementAt(from)).Name();
	Vector v = (Vector)checkTrans.get(fname);
	Point p = ((State)states.elementAt(to)).getPoint();
	
	return v.indexOf(p) != -1;
    }

    public boolean TransAlreadyMade(int from, int to){
    //postcond:	returns true if the user has already added the transition
	Point p = ((State)states.elementAt(to)).getPoint();

	return ((State)states.elementAt(from)).hasTransition(p);
    }

    public void mousePressed(MouseEvent e)
    {	
        if(dragging || moving)
           return;

	current = find(e.getX(), e.getY());
	if (current < 0)
        {    //not inside a circle
	    selected = -1;
	}
	else if(e.getClickCount() == 1)
        {
           int mods = e.getModifiers();
           if((mods & Event.ALT_MASK) != 0 ||
              (mods & Event.SHIFT_MASK) != 0 ||
              (mods & InputEvent.BUTTON2_MASK) != 0)
           {   
              //middle mouse button
              moving = true;
           }
           else if((mods & Event.META_MASK) != 0 ||
                   (mods & Event.CTRL_MASK) != 0 ||
	           (mods & InputEvent.BUTTON3_MASK) != 0)
           {
              //right mouse button
              if(showTerms)
                 addDoubleCircle(current);
	      selected = -1;
           }
           else
           {
              //left mouse button and anything else
              dragging = true;
              dragPoint = new Point(e.getX(), e.getY());
	      Select(current);
           }
	}
	repaint();
    }

    public void mouseReleased(MouseEvent e)
    {
       current = find(e.getX(), e.getY());
       if(current >= 0 && dragging)
       {
          Select(current);
       }
       selected = -1;
       dragging = false;
       moving = false;
       wasMoved = false;
       repaint();
    }

    public void mouseClicked(MouseEvent e)
    {}

    public void mouseEntered(MouseEvent e)
    {}

    public void mouseExited(MouseEvent e)
    {}

    public void mouseMoved(MouseEvent e)
    {
	if(find(e.getX(), e.getY()) >= 0)
	    setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
	else
	    setCursor(Cursor.getDefaultCursor());
    }

    public void mouseDragged(MouseEvent e)
    {
        Dimension d = getSize();
        int x = e.getX();
        int y = e.getY();
        if(dragging)
        {
           //buffer the image drawing
           Image temp = createImage(d.width, d.height);
           Graphics g = temp.getGraphics();
           g.setFont(new Font("TimesRoman", Font.PLAIN, 10));
           
           drawTransitions(g);
           g.drawLine(dragPoint.x, dragPoint.y, x, y);
           drawStates(g);
           getGraphics().drawImage(temp, 0, 0, null);
           return;
        }
        else if(moving)
        {
           if(current >= 0)
           {
              Graphics g = getGraphics();
              g.setXORMode(getBackground());
              repaint();
              
              if(x >= d.width)
                 x = d.width - CIRCLERAD/2;
              if(y >= d.height - bottom)
                 y = d.height - bottom - CIRCLERAD/2;
              if(x < 0)
                 x = 10;
              if(y < top)
                 y = 30;
              
              ((State)states.elementAt(current)).move(x, y);
              repaint();
              g.dispose();
              selected = -1;
              wasMoved = true;
           }
        }
    }

    public boolean doneVisualizing = false;
    private boolean dragging = false;
    private boolean moving = false;
    private Point dragPoint;

    private static final int CIRCLERAD = 25;
    private static final int MAXCIRCLES = 100;

    private Vector states = new Vector();
    private int nStates = 0;
    //vector of states that have transitions;
    private Vector transitions = new Vector();
    private int nTransitions = 0;
    private int current = -1;
    private int selected = -1;
    private int loopW = 20;
    private int loopH = 50;
    private int Tside = 10;
    private int Theight = 8;
    private int addLength = 30;
    private Image offscreen = null;
    private Graphics offgraph = null;
    private int bottom = 30;
    private int top = 20;
    private Grammar graph;
    private int termPix = 4;
    private boolean init = false;
    private boolean wasMoved = false;
    private Hashtable checkTrans = new Hashtable(151);
    private int totalTrans = 0;
    private int userTrans = 0;
    private boolean showTerms;
    private Vector nonTermVars;
    private Hashtable checkTerms = new Hashtable(151);
    private int totalTerms = 0;
    private int	userTerms = 0;
}
