/*---------------------------------------------------------------------------
File:					Pate.java
Package:				PateLib  Version 1.0
Author:					Anna O. Bilska  V1.0 08/07/96
                                
Description of Contents:       
				
--------------------------------------------------------------------------*/

/* 
 * Susan H. Rodger, Anna O. Bilska , Jason R. Salemme
 * Computer Science Department
 * Duke University
 * August 1996
 * Supported by National Science Foundation DUE-9596002 and DUE-9555084.
 *
 * Copyright (c) 1996
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

// a class which sets up the frame and user environment
// for the parser

// *****************************************************
// ************METHOD INDEX*****************************
// *****************************************************
// CONSTRUCTORS:
// public ParserEnvironment(Grammar gramm,String filename)
// METHODS:
// private void add(Component c, GridBagLayout gbl,
//		GridBagConstraints gbc, int x,int y,int w, int h,
//		int wx, int wy)
// private void SetUpMenu()
// public void actionPerformed(ActionEvent evt)
// public void Help()
// private void About()
// private void Parse()
// private void PauseParser()
// private void StopParserThreads()
// private boolean IsLegal(String w)
// private boolean DerivationExists()
// private  boolean SaveDerivation()

public class ParserEnvironment extends Frame 
    implements ActionListener, ItemListener
{
 
    public ParserEnvironment(Grammar gramm,String filename)
	// constructor
	// gramm: the grammar
	// filename: name of the grammar file opened
    {
	g = gramm;
	setTitle("Brute Parser: "+filename);
	SetUpMenu();
	GridBagLayout gbl = new GridBagLayout();
        Panel p = new Panel(gbl);
	setLayout(new BorderLayout());
        add(p, "Center");
	GridBagConstraints gbc = new GridBagConstraints();
        setBackground(Params._pe_backcolor);
		
	wordField = new TextField(15);
	Label wordLabel = new Label("String to parse:",Label.RIGHT);
	myScrollText = new ScrollCanvas(330,400);
	myScrollGraph = new ScrollCanvas(330,400);

	// set up the canvas which imitates 
	// the depressed Parser button	
	SmilyCanvas = new Canvas();
	SmilyCanvas.setSize(30,30);
	SmilyCanvas.setBackground(Color.lightGray);
	SmilyCanvas.setVisible(true);
		
	// set up the input panel where the string is entered
	Panel InputPanel = new Panel();
	InputPanel.setLayout(new FlowLayout());
	InputPanel.add(wordLabel);
	InputPanel.add(wordField);
	Button b;
	InputPanel.add(b = new Button("Clear"));
	b.addActionListener(this);
		
	// set up a choice to allow the user to choose
	// the kind of output	
	OutputChoice = new Choice();
	OutputChoice.addItem(TEXTUAL);
	OutputChoice.addItem(GRAPHICAL);
	OutputChoice.addItemListener(this);
	OutputChoice.setVisible(true);
	Panel OutputChoicePanel = new Panel();
	OutputChoicePanel.setLayout(new FlowLayout());
	OutputChoicePanel.add(OutputChoice);

	// add the input panel to the frame
	gbc.insets = new Insets(10,3,3,15);
	add(p, InputPanel,gbl,gbc,0,0,6,1,1,1);

	// add the parse buttons to the frame
	gbc.fill = GridBagConstraints.BOTH;
	gbc.insets = new Insets(5,15,5,5);
	ParseButton = new Button(" Parse ");
	ParseButton.addActionListener(this);
	PauseButton = new Button(" Pause Parser ");
	PauseButton.addActionListener(this);
	add(p, ParseButton,gbl,gbc,0,1,3,1,1,1);
	add(p, SmilyCanvas,gbl,gbc,0,1,3,1,1,1);
	gbc.insets = new Insets(5,5,5,15);
	add(p, PauseButton,gbl,gbc,3,1,3,1,2,1);

	// add the choice panel to frame
	gbc.insets = new Insets(5,15,0,15);
	gbc.fill = GridBagConstraints.NONE;
	add(p, OutputChoicePanel,gbl,gbc,0,2,6,1,2,2);
	gbc.insets = new Insets(1,1,1,1);
		
	// add the ouput scroll canvases to frame
	gbc.fill = GridBagConstraints.BOTH;
	gbc.insets = new Insets(0,15,15,15);
	add(p, myScrollText,gbl,gbc,0,3,6,8,100,100);
	add(p, myScrollGraph,gbl,gbc,0,3,6,8,100,100);
	myScrollText.setVisible(true);
	gbc.fill = GridBagConstraints.NONE;

        p = new Panel(new BorderLayout());
        Panel p2 = new Panel(new FlowLayout());
        p2.add(ShowButton = new Button("Show All"));
        ShowButton.setEnabled(false);
        ShowButton.addActionListener(this);
        p2.add(PrevButton = new Button("Show Previous"));
        PrevButton.setEnabled(false);
        PrevButton.addActionListener(this);
        p2.add(NextButton = new Button("Show Next"));
        NextButton.setEnabled(false);
        NextButton.addActionListener(this);
        p.add(p2, "Center");
        p.add(myRuleLabel = new Label("     ", Label.CENTER), "North");
        add(p, "South");

        validate();
	pack();
	myScrollText.resizeScrollBars();
	myScrollGraph.resizeScrollBars();
	oldSize = this.getSize();
	addWindowListener(
	    new WindowAdapter()
	    {
		public void windowClosing(WindowEvent e)
		{
		    ParserEnvironment.this.dispose();
		}
	    }
	);
    }

    private void add(Panel p, Component c, GridBagLayout gbl,
		GridBagConstraints gbc, int x,int y,int w, int h,
		int wx, int wy)
	// postcondition: adds a component to the gridbaglayout
    {
       gbc.gridx = x;
       gbc.gridy = y;
       gbc.gridwidth = w;
       gbc.gridheight = h;
       gbc.weightx = wx;
       gbc.weighty = wy;
       gbl.setConstraints(c,gbc);
       p.add(c);
    }	

    private void SetUpMenu()
	// postcondition: the menu is setup
    {
		String temp;
		MenuBar mbar = new MenuBar();
		Menu m;

		MenuItem mi;
		m = new Menu("File");
		m.add(mi = new MenuItem("Save Derivation As"));
		mi.addActionListener(this);
		MenuItem pr = new MenuItem("Print to Sun Printer");
		pr.addActionListener(this);
		m.add(pr);
		m.addSeparator();
		m.add(mi = new MenuItem("Close"));
		mi.addActionListener(this);
		mbar.add(m);
		pr.setEnabled(false);
	
		m = new Menu("Options");
		duplicatesCheck = new CheckboxMenuItem("Ignore Duplicate Strings");
		duplicatesCheck.setState(true);
		duplicatesCheck.addActionListener(this);
		m.add(duplicatesCheck);
		mbar.add(m);
	
		// put the grammar in the menu for the user to see
		m = new Menu("GrammarPeek");
		for (g.FirstProd(); !g.ProdsDone(); g.NextProd())
		{
			temp = g.CurrentLeftProd() + "->" + g.CurrentRightProd();
			m.add(mi = new MenuItem(temp));
			mi.addActionListener(this);
		}
		mbar.add(m);

		m = new Menu("Help");
		m.add(mi = new MenuItem("Help HTML"));
		mi.addActionListener(this);
		m.add(mi = new MenuItem("Help Text"));
		mi.addActionListener(this);
		m.addSeparator();
		m.add(mi = new MenuItem("About..."));
		mi.addActionListener(this);
		mbar.add(m);

		setMenuBar(mbar);	
    }	

    public void itemStateChanged(ItemEvent e)
	///event for list item
    {
	String arg = OutputChoice.getSelectedItem();
	if (arg.equals(TEXTUAL))
	{
	    myScrollGraph.setVisible(false);
	    myScrollText.setVisible(true);

	    if (((this.getSize()).width !=(oldSize.width))
		    || (this.getSize()).height!=(oldSize.height))
	    {
		//System.out.print("different size ");
		//System.out.print(oldSize.width+" "+oldSi
		oldSize = this.getSize();
		validate();
	    }
	}
	else if (arg.equals(GRAPHICAL))
	{
	    myScrollText.setVisible(false);
	    myScrollGraph.setVisible(true);
	    if(((this.getSize()).width !=(oldSize.width))
		    || (this.getSize()).height!=(oldSize.height))
	    {
		//System.out.println("different size");
		oldSize = this.getSize();
		validate();
	    }
		
	}
	return;
    }
    
    public void actionPerformed(ActionEvent evt)
	// the action handling function
    {
	String arg;
	if(evt.getSource() instanceof Button)
	{
	    arg = ((Button)evt.getSource()).getLabel();
	    if(arg.equals("Clear"))
	    {
               wordField.setText("");
	    }
	    else if(arg.equals(" Parse "))
	    {
               StopParserThreads();
               ParseButton.setVisible(false);
               SmilyCanvas.setVisible(true);
               Parse();
	    }
	    else if(arg.equals(" Pause Parser "))
	    {
               PauseParser();
	    }
            else if(evt.getSource() == ShowButton)
            {
               while(myScrollGraph.nextRule())
               {
               }

               while(myScrollText.nextRule())
               {
               }

               NextButton.setEnabled(false);
               myRuleLabel.setText("");
               myScrollGraph.repaintUpdate();
               myScrollText.repaintUpdate();
            }
            else if(evt.getSource() == NextButton)
            {
               NextButton.setEnabled(myScrollGraph.nextRule());
               myScrollText.nextRule();
               PrevButton.setEnabled(true);
               myRuleLabel.setText("Rule Used: " + 
                  myScrollGraph.getCurrentRule());
               myScrollGraph.repaintUpdate();
               myScrollText.repaintUpdate();
            }
            else if(evt.getSource() == PrevButton)
            {
               PrevButton.setEnabled(
                  myScrollGraph.prevRule());
               myScrollText.prevRule();
               NextButton.setEnabled(true);
               String str = myScrollGraph.getCurrentRule();
               if(str.equals(""))                  
                  myRuleLabel.setText("");
               else 
                  myRuleLabel.setText("Rule Used: " + str);
               myScrollGraph.repaintUpdate();
               myScrollText.repaintUpdate();
            }
	    return;
	}
	else if(evt.getSource() instanceof MenuItem)
	{
	    arg = ((MenuItem)evt.getSource()).getLabel();
	    if(arg.equals("Close"))
	    {
               StopParserThreads();
               dispose();
	    }
	    else if(arg.equals("Ignore Duplicate Strings"))
	    {
               ignoreduplicate = duplicatesCheck.getState();
	    }
	    else if(arg.equals("About..."))
	    {
               About();
	    }
	    else if(arg.equals("Help HTML"))
	    {
		HelpWindow help = HelpWindowHTML.getHelpWindow("PATE",
		    Params._hd_bkcolor);
		help.setVisible(true);
	    }
	    else if(arg.equals("Help Text"))
	    {
		HelpWindow help = HelpWindowText.getHelpWindow("PATE",
		    Params._hd_bkcolor);
		help.setVisible(true);
	    } 
	    else if(arg.equals("Save Derivation As"))
	    {
		if (DerivationExists())	
		{
                   SaveDerivation();
		}
		else
		{
                   OKDialog d = new OKDialog("Derivation does not exist.\n  Cannot save.  ",this,"Error");
                   d.showResize();
		}
	    }
	    return;
	}
    }


	private void About()
	// postcondition: a dialog box which contains
	// info about the program is displayed
	{
		String mesg = new String("BruteParser V1.0 \n August 1996 \n (c)Susan H. Rodger \n Anna O.Bilska \n");
		AboutDialog a = new AboutDialog(this, Params.parserPic, mesg,
				Params.aboutHeight, Params.aboutWidth);
		a.show();
	}

	private void Parse()
	// postcondition: the parser is started
	{	
		String word = new String();
		ignoreduplicate = duplicatesCheck.getState();
		word = wordField.getText();
		if (IsLegal(word))
		{
			parsley = new Parser(g,ignoreduplicate);
			Smily = new SmilyIndicator(SmilyCanvas);
			monitor = new ParserMonitor(
                           g,
                           word,
                           parsley,
                           myScrollText,
                           myScrollGraph,
                           this,
                           Smily,ParseButton);
			parsley.SearchFor(word);
			parsley.start();
			Smily.setPriority(Thread.NORM_PRIORITY - 3);
			Smily.start();
			monitor.start();
		}
		else
		{
			OKDialog d = new OKDialog("Illegal string. \n  Cannot parse. \n(You can enter terminals \n from the grammar only)", this,"Error");
			d.showResize();
			ParseButton.setVisible(true);
		}
	}

	private void PauseParser()
	// postcondition: if the parser is working
	// it is paused, and then either stopped
	// or resumed, depending on the user input
	{
		if (parsley!=null)
		{
			if (!parsley.isAlive())
			{
                           OKDialog dd = new OKDialog("Cannot PAUSE. \n  Parser is not running.", this,"Sorry");
                           dd.showResize();
                           return;
			}
			parsley.PausePConstant();
			Smily.safeSuspend();
			
			//System.out.println("suspended :"+parsley.GetIterations() + "\n");	

			String mesg = new String();
			mesg = "There are curently " + parsley.GetIterations() +
						" \n nodes in the derivation tree.\n Do you wish to continue? "; 
			TextInDialog d = new TextInDialog(mesg, PAUSE_BUTTONS, this, 
								"Parser is Paused");
			d.showResize();
			String choice = d.getPushedButton();
			if (choice.equals(CONTINUE_PARSING))
			{
				if (parsley.isAlive())
				{
					parsley.UnPausePConstant();
					Smily.safeResume();
					//System.out.println("resumed: " + parsley.GetIterations() + "\n");
				}
			}
			else if (choice.equals(STOP))
			{
				StopParserThreads();
				//System.out.println("stopped: " + parsley.GetIterations() + "\n");
			}
		}
	}


	private void StopParserThreads()
	// postcondition: stops the threads that 
	// run the parser and with the parser
	{
           if(parsley != null)
           {
              parsley.safeStop();
              parsley.UnPausePConstant();
              parsley.CleanUp();
           }
           if (Smily != null)
           {
              Smily.safeStop();
              Smily = null;
           }
           ParseButton.setVisible(true);
	}

	private boolean IsLegal(String w)
	// postcondition: returns true if the string 
	// to be parsed contains only terminals that
	// are part of the grammar 
	{
		int len = w.length();
		if(len==0)
                   return false;
		for (int i=0; i<len; i++)
		{
			String s = String.valueOf(w.charAt(i));
			s = w.substring(i,i+1);;
			if (!g.IsInTerms(String.valueOf((w.charAt(i)))))
				return false;
		}
		return true;
	}


	private boolean DerivationExists()
	// postcondition: returns true if the parser has
	// found the derivation
	{
		if (parsley!=null)
		// if the parser exists
		{
			if (parsley.IsDoneParsing() && parsley.IsFound())
			// and has found the derivation
			{
				// stop just in case
				StopParserThreads();
				return true;
			}
			return false;
		}
		return false;
	}

	private boolean SaveDerivation()
	// preconditon: derivation exists
	// postcondition: saves the derivation to 
	// a file
	{
                if(!parsley.isDone())
                {
                   OKDialog dg = new OKDialog("Cannot Save:\nParser is not done!", this, "Error");
                   dg.showResize();
                   return false;
                }
		FileDialog fd = new FileDialog(this,"Save Derivation" ,FileDialog.SAVE);
		fd.setDirectory(".");
		fd.show();
		String File = fd.getFile();
		String Dir = fd.getDirectory();
		if (Dir == null || File == null)
                   return false;
		String fName = Dir + File;
	
		words = parsley.GetDerivation();
		prods = parsley.GetPathProds();

		DerivationSaver ds = new DerivationSaver(words,prods,g);
		try
		{
                   if (!fName.endsWith(".drv"))
                      fName += "." + "drv";
                   ds.write(fName);
                   return true;
		}
		catch (IOException e) 
		{
                   OKDialog dg = new OKDialog("I/O Error.\nFile not saved.\n"+e.getMessage(), this, "Error");
                   dg.showResize();
                   return false;
		}
		catch (NoSuchElementException e) 
		{
                   OKDialog dg = new OKDialog("Tokenizer Error.\nFile not saved.\n"+e.getMessage(), this, "Error");
                   dg.showResize();
                   return false;
		}
		catch (Exception e) 
		{
                   OKDialog dg = new OKDialog("File Error.\nFile not saved.\n"+e.getMessage(), this, "Error");
                   dg.showResize();
                   return false;
		}
        }

        public void setFinishParsing()
        {
            //update the view
            itemStateChanged(null);
            ShowButton.setEnabled(true);
            NextButton.setEnabled(true);
        }

	public PositionTree posTree = new PositionTree();
   
	private TextField wordField;
	private Grammar g = new Grammar();
	private Parser parsley;
	private ParserMonitor monitor;
	private CheckboxMenuItem duplicatesCheck;
	private boolean ignoreduplicate=false;

	final static String CONTINUE_PARSING = "Continue Parsing";
	final static String STOP = "Stop";
	final static String[] PAUSE_BUTTONS = { CONTINUE_PARSING, STOP };
	final static String GRAPHICAL = "Derivation: Graphical Output";
	final static String TEXTUAL = "Derivation: Text Output";
	private String pushedB = "";

	private SmilyIndicator Smily;
	private Button ParseButton;
	private Button PauseButton;
        private Button ShowButton;
        private Button NextButton;
        private Button PrevButton;
        private Label  myRuleLabel;
	private Canvas SmilyCanvas;
	private ScrollCanvas myScrollText;
	private ScrollCanvas myScrollGraph;
	private Choice OutputChoice;
	private Vector words = new Vector();
	private Vector prods = new Vector();
	private Dimension d;
	private Dimension oldSize;
} 




