/*---------------------------------------------------------------------------
File:					Pate.java
Package:				PateLib  Version 1.0
Author:					Anna O. Bilska (Anya) V1.0 08/07/96        
--------------------------------------------------------------------------*/

/* 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme
 * Computer Science Department
 * Duke University
 * August 1996
 * Supported by National Science Foundation DUE-9596002 and DUE-9555084.
 *AddRuleDialog.class	    ParserMonitor.java		TransUnit.java

 * Copyright (c) 1996
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

package PateLib;

import java.awt.*;
import java.util.*;

// written by Anya Bilska, summer 1996

// The following class is a  parser monitor thread, 
// which runs at the same time as the parser,
// and checks whether the parser is done. 
// If so, and the word was found, it prints out 
// the resulting derivation.

// ***************************************************
// ***METHOD INDEX************************************
// ***************************************************
// CONSTRUCTOR:
// public ParserMonitor(Grammar g, String w,
//				Parser p, TextArea printoutarea,
//				ScrollCanvas grapharea,
//				Frame par, SmilyIndicator smily)
// METHODS:
// public void run()
// public synchronized void PrintDerivation()
// public synchronized void PauseParser()
// private synchronized int FindLongestLine(String mesg)
// private void StopParserThreads()


public class ParserMonitor extends Thread
{	
	public boolean isStopped;

	public ParserMonitor(Grammar g, String w,
					Parser p, ScrollCanvas printoutarea,
					ScrollCanvas grapharea,
					ParserEnvironment par, SmilyIndicator smily,
					Button parseBut)
	// constructor
	// g: grammar used by the parser, needed for output
	// w: word looked for, needed for output
	// p: the parser thread
	// printoutarea: the area used for output
	// par: the parent frame
	// smily: smily thread
	{	
		myG = g; 	 
		myWord = w;
		// myParser: the parser thread
		myParser = p;
		myText = printoutarea;
		myGraph = grapharea;
		parent = par;
		mySmily = smily;
		myParseButton = parseBut;
		isStopped = false;
	}

	public void run()
	{
		int i=1;
		while(!myParser.isDone())
		{
                   if (((i%1000)==0) && !myParser.IsPaused())
                      PauseParser();
                   if(isStopped)
                      break;
                   try
                   {
                      Thread.sleep(30);
                   }
                   catch(InterruptedException e){}
                   i++;
		}
		mySmily.safeStop();
		mySmily = null;
		if (myParser.IsDoneParsing() && myParser.IsFound())
		{
                   PrintDerivation();
		}	
		else if (!myParser.IsDoneParsing())
		{
                   String mesg = "parser was stopped, word not found, \nThe derivation tree contained " +myParser.GetIterations()+" nodes. "; 
                   //myText.setSize(mesg.length()*10,40);
                   //myGraph.setSize(mesg.length()*10,40);
                   myText.MessageToCanvas(mesg);
                   myGraph.MessageToCanvas(mesg);
		}
		else
		{
                   String mesg = myWord + " is not part of language, \nThe derivation tree contained " +myParser.GetIterations()+" nodes. "; 
                   //myText.setSize(mesg.length()*10,40);
                   //myGraph.setSize(mesg.length()*10,40);
                   myText.MessageToCanvas(mesg); 
                   myGraph.MessageToCanvas(mesg);
		}		
		myParseButton.setVisible(true);
		myText.setVisible(false);
		myGraph.setVisible(false);
                parent.setFinishParsing();
	}




	private synchronized void PrintDerivation()
	// precondition: the word was found
	// postcondition: the derivation is printed to the outputarea
	{
		int h, w;
                int msgheight;
		
		words = myParser.GetDerivation();
		myProdsUsed = myParser.GetPathProds();
		String mesg = (myWord +" is part of language. \nThe derivation tree contained " +myParser.GetIterations()+" nodes. "); 
	        StringTokenizer st = new StringTokenizer(mesg,"\n");
		FontMetrics fm = myText.getGraphics().getFontMetrics();
                msgheight = (st.countTokens() + 10) * fm.getHeight();

		PathTree p = new PathTree(myG,myProdsUsed);
		myTree = p.PathTreeFromDerivation();

		h = p.getHeightNeeded() + msgheight;
		w = p.getWidthNeeded();
		if (w<260)
		    w=260;
		
		myGraph.setSize(w,h);
		myGraph.MessageToCanvas(mesg);
		myGraph.DerivationToCanvas(myProdsUsed, myG, myTree);

		
		h = words.size() * fm.getHeight() + msgheight + 100;

		w = FindLongestLine(mesg);
		if (w<280)
		    w=280;
		myText.setSize(w,h);

		myText.MessageToCanvas(mesg);
		myText.DerivationToCanvas(words,myProdsUsed,myG);
	
	}

	private synchronized int FindLongestLine(String mesg)
	// postcondtion: finds the length of the longest
	// line to be outputed (for textual output)
	{
	
		int maxLength = 0;
		int len = words.size();
		for (int i=0; i<len; i++) 
		{
			if (maxLength<((String)words.elementAt(i)).length())
				maxLength = ((String)words.elementAt(i)).length();
		}
	
		int temp=0;
		int maxL=0;
		for (myG.FirstProd(); !myG.ProdsDone(); myG.NextProd())
		{
			temp= (myG.CurrentRightProd()).length() +
				  (myG.CurrentLeftProd()).length();
			if (temp>maxL)
				maxL = temp;
		}

		maxLength += (maxL+6);
	
		if (200 > maxLength*10)
			return (200);
		return (maxLength*10);
	}

	public synchronized void PauseParser()
	// postcondition: the parser is paused and then either
	// stopped or resumed, depending on the user input
	{
		if (myParser!=null)
		{
			myParser.PausePConstant();
			mySmily.safeSuspend();

			String mesg = new String();
			mesg = "There are currently " + myParser.GetIterations() + " \n nodes in the tree.\n It is POSSIBLE that the \n word is not in the language or \n that it would take too long to parse.  \n Do you wish to continue? "; 
			TextInDialog d = new TextInDialog(
                           mesg,
                           PAUSE_BUTTONS,
                           parent, 
                           "Pause Parser?", 200, 240);
			d.showResize();
			String choice = d.getPushedButton();
			if (choice.equals(CONTINUE_PARSING))
			{
                           if (myParser.isAlive())
                           {
                              myParser.UnPausePConstant();
                              mySmily.safeResume();
                           }
			}
			else if (choice.equals(STOP))
			{
                           StopParserThreads();
			}
		}
	}

	private synchronized void StopParserThreads()
	// postcondition: stops the threads that 
	// run the parser and with the parser
	{
		if (myParser !=null)
		{
			myParser.safeStop();
			myParser.UnPausePConstant();
			//myParser.stop();
			myParser.CleanUp();
			//myParser = null;
		}
		if (mySmily != null)
		{
			//mySmily.stop();
		    mySmily.safeStop();
		    //mySmily = null;
		}
		myParseButton.setVisible(true);
		isStopped = true;
	}



	private String myWord;
	private Grammar myG;
	private Vector myProdsUsed;
	private Vector words;
	private Parser myParser;
	private ScrollCanvas myText;
	private ScrollCanvas myGraph;
	private ParserEnvironment parent;
	private Vector myTree;
	private boolean isGraph;

	final static String CONTINUE_PARSING = "Continue Parsing";
	final static String STOP = "Stop";
	final static String[] PAUSE_BUTTONS = { CONTINUE_PARSING, STOP };
	private String pushedB = "";
	private SmilyIndicator mySmily;
	private Button myParseButton;
}
