package PateLib;

import java.util.*;
import java.awt.*;

public class PathTree
{
   private Grammar myG;
   private Vector myPathGuide;
   private TreeNode myTreeHead;
   private Shifter shiftee = new Shifter();
   private Vector myLevelNodes;
   private int adjustWidth = 0;

   public PathTree(Grammar g, Vector p)
   {
      myG = g;
      myPathGuide = (Vector) p.clone();
      myLevelNodes = new Vector();
   }

   /**
    * builds a tree from the given grammar and rules and returns all of
    * the nodes in a vector for easy drawing
    */
   public Vector PathTreeFromDerivation()
   {
      int temp, index, prod;
      int curlevel = 0;
      TreeNode node;

      myTreeHead = new TreeNode("S", new Vector(), null, null);
      myTreeHead.setRuleNum(curlevel++);
      int len = myPathGuide.size();
      if(!myG.IsUnrestricted())
      {
         for(int i = len-2; i>=0; i--)
         {
            temp = ((Integer)myPathGuide.elementAt(i)).intValue();
            index = shiftee.GetIndex(temp);
            prod = shiftee.GetProd(temp);
            node = findLeafNode(index);
            setNodeChildren(node, myG.RightProdAt(prod), curlevel++);
         }
      }
      else
      {
         for(int i = len-2; i>=0; i--)
         {
            temp = ((Integer)myPathGuide.elementAt(i)).intValue();
            index = shiftee.GetIndex(temp);
            prod = shiftee.GetProd(temp);
            if(myG.LeftProdAt(prod).length() > 1)
            {
               node = createUnrestrictedLeafNode(
                  myG.LeftProdAt(prod),
                  index,
                  curlevel);
            }
            else
            {
               node = findLeafNode(index);
            }
            
	    setNodeChildren(node, myG.RightProdAt(prod), curlevel++);
         }
      }

      createLevelNodes();
      setUpCoordinates();
      Vector retVect = new Vector();
      retVect.addElement(myTreeHead);
      createVector(retVect, myTreeHead);
      return retVect;
   }

   private void createVector(Vector vect, TreeNode head)
   {
      if(head == null)
         return;

      TreeNode temp;
      for(Enumeration e = head.children.elements(); e.hasMoreElements(); )
      {
         temp = (TreeNode) e.nextElement();
         vect.addElement(temp);
         createVector(vect, temp);
      }
   }

   public int getWidthNeeded()
   {
      return (getWidestLevel() + 1) * TreeNode.diameter + adjustWidth + 100;
   }

   public int getHeightNeeded()
   {
      return (int)(myLevelNodes.size() * (2 * TreeNode.diameter));
   }

   /**
    * returns the greatest number of nodes for a given level
    */
   private int getWidestLevel()
   {
      int retSize = 0;
      Vector level;

      for(Enumeration e = myLevelNodes.elements(); e.hasMoreElements(); )
      {
         level = (Vector) e.nextElement();
         if(level.size() > retSize)
         {
            retSize = level.size();
         }
      }

      return retSize;
   }

   /**
    * sets up the coordinates for all of the nodes
    */
   private void setUpCoordinates()
   {
      int width = (getWidestLevel() + 1) * TreeNode.diameter;
      int startX = 20;
      int startY = 75;
      int interval;
      TreeNode lastParent = null;

      adjustWidth = 0;
      interval = width / 3;
      Position pos = new Position(0, 0);
      pos.x = interval + startX;
      pos.y = startY;
      myTreeHead.coordinate = new Position(pos.x, pos.y);

      Vector vect;
      TreeNode curnode;
      for(Enumeration lev = myLevelNodes.elements(); lev.hasMoreElements(); )
      {
         vect = (Vector) lev.nextElement();
         interval = width / (vect.size() + 1);
         pos.x = startX;
         for(Enumeration e = vect.elements(); e.hasMoreElements(); )
         {
            curnode = ((TreeNode) e.nextElement());
            if(curnode.group != null)
            {
		curnode.coordinate = new Position(
                  (((TreeNode) curnode.group.firstElement()).coordinate.x +
                  ((TreeNode) curnode.group.lastElement()).coordinate.x) / 2,
                  ((TreeNode) curnode.group.firstElement()).coordinate.y); 
                lastParent = null;
            }
            else
            {
                if(lastParent != curnode.parent && interval <= TreeNode.diameter)
                {
                   pos.x += TreeNode.diameter + 10;
                }
                else
                {
                   pos.x += interval;
                }

		//straighten out some lines
		if(curnode.parent != null && curnode.parent.children.size() == 1)
		{
		    if(pos.x < curnode.parent.coordinate.x)
                    {
                       adjustWidth += curnode.parent.coordinate.x - pos.x;
                       pos.x = curnode.parent.coordinate.x;
                    }
                }
                if(curnode.parent != null && pos.x > curnode.parent.coordinate.x &&
                       vect.firstElement() == curnode)
                {
                   pos.x = curnode.parent.coordinate.x;
                }
                curnode.coordinate = new Position(pos.x, pos.y);
                lastParent = curnode.parent;
	    }
         }

         pos.y += 2 * TreeNode.diameter;
      }
   }

   /**
    * stores the level values into the Vector myLevelNodes
    */
   private void createLevelNodes()
   {
      Vector vect = new Vector();
      int x = 0;
      getLevelNodes(vect, x, myTreeHead);
      while(vect.size() != 0)
      {
         myLevelNodes.addElement(vect);
         vect = new Vector();
         getLevelNodes(vect, ++x, myTreeHead);
         
      }
   }

   /**
    * finds all of the nodes at a specified depth from the head
    * and returns them in the vector
    */
   private void getLevelNodes(Vector vect, int x, TreeNode head)
   {
      if(x == 0)
      {
         vect.addElement(head);
         return;
      }

      for(Enumeration e = head.children.elements(); e.hasMoreElements(); )
      {
         getLevelNodes(vect, x - 1, (TreeNode) e.nextElement());
      }
   }

   /**
    * finds all of the nodes at that are terminals
    * and returns them in the vector in order
    */
   private void getTerminalNodes(Vector vect, TreeNode head)
   {
      if(head.children.size() == 0 && !head.isGrouped)
      {
         vect.addElement(head);
         return;
      }

      for(Enumeration e = head.children.elements(); e.hasMoreElements(); )
      {
         getTerminalNodes(vect, (TreeNode) e.nextElement());
      }
   }

   /**
    * finds the level of the node in respect to the top of the tree
    */
   private int getLevel(TreeNode head)
   {
      int level = 0;
      while(head != null)
      {
         head = head.parent; 
	 ++level;
      }

      return level;
   }

   /**
    * finds where an unrestricted grammar node should be created
    * and create the node and returns it
    */
   private TreeNode createUnrestrictedLeafNode(
      String identifier,
      int index, int rulenum)
   {
      Vector terminals = new Vector();
      getTerminalNodes(terminals, myTreeHead);
      Vector group = new Vector();
      TreeNode node = null;

      for(int x = 0; x < identifier.length(); ++x)
      {
         group.addElement(terminals.elementAt(index + x));
      }

      //failed to find unrestricted grammar
      if(group.size() == 0)
         return null;

      //find the deepest branch
      int deepest = 0;
      for(Enumeration e = group.elements(); e.hasMoreElements(); )
      {
         node = (TreeNode) e.nextElement();
         if(getLevel(node) > deepest)
            deepest = getLevel(node);
      }

      //move all of the leafs onto the same level
      for(Enumeration e = group.elements(); e.hasMoreElements(); )
      {
         node = (TreeNode) e.nextElement();
         while(getLevel(node) < deepest)
         {
            node = moveTerminalDown(node);
            node.setRuleNum(rulenum);
         }
      }

      TreeNode retNode = new TreeNode("group", new Vector(), null, null);

      //move all of the group pointers down to the bottom level
      //and add the group node to their children lists
      for(int loop = 0; loop < group.size(); ++loop)
      {
         while(((TreeNode) group.elementAt(loop)).children.size() != 0)
         {
            group.setElementAt(
               ((TreeNode) group.elementAt(loop)).children.elementAt(0),
               loop);
         }
         ((TreeNode) group.elementAt(loop)).isGrouped = true;
      }

      node = (TreeNode) group.lastElement();
      int pos = node.parent.children.indexOf(node);
      node.parent.children.insertElementAt(retNode, pos + 1);
      retNode.parent = node.parent;
      retNode.group = group;
      retNode.setRuleNum(rulenum);
      return retNode;
   }

   /**
    * moves a terminal down one level by placing an empty node on current level
    * returns the new node on the next level
    */
   private TreeNode moveTerminalDown(TreeNode node)
   {
      int num = node.getRuleNum();
      TreeNode newnode = new TreeNode(node.data, new Vector(), node, null);
      node.children.addElement(newnode);
      node.data = "";

      //force it to look at its deepest child for what rule to show at
      newnode.setRuleNum(num);
      return newnode;
   }

   /**
    * finds a leaf node with the given identifier
    */
   private TreeNode findLeafNode(int index)
   {
      Vector terms = new Vector();
      getTerminalNodes(terms, myTreeHead);
      return (TreeNode) terms.elementAt(index);
   }

   /**
    * sets the children value for the specified node given the
    * string to parse
    */
   private void setNodeChildren(TreeNode node, String str, int level)
   {
      if(node == null)
         return;

      int end;
      String id;
      int x = 0;
      TreeNode temp;
      while(x < str.length())
      {
         end = myG.getEnd(str, x);
         id = str.substring(x, end);
         node.children.addElement(
            temp = new TreeNode(id + "", new Vector(), node, null));
         temp.setRuleNum(level);
         x = end;
      }
   }
}




