package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/*
---------------------------------------------------------------------------
File	    :		ProdEnvironment.java
Package	    :		PateLib  Version 1.0
Author	    :		Jason Robert Salemme
Programmed  :		8 Aug 96	
Modified    :           Ted Hung 8/99
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme, Ted Hung
 * Computer Science Department
 * Duke University
 * August 1999
 * Supported by National Science Foundation DUE-9596002, DUE-9555084,
 *  and DUE-9752583.
 *
 * Copyright (c) 1999
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * The ProdEnvironment class is the starting point for Pate.  It is part of the
 * main application so this is really not a class for one to use in any
 *significant way.
 */
public class ProdEnvironment extends Frame implements ActionListener
{
    private int myNumLines;
    private Vector myLeftFields;
    private Vector myRightFields;
    private Panel myScrollPanel;

    private GridLayout myFieldGrid;
    private Panel myFieldPanel;
    private GridLayout myNumberGrid;
    private Panel myNumberPanel;

    private String filename;	//filename of grammar
    private SaveText Save;	//method for saving

    public Button button_1;
    public Button button_2;

    final static String SAVE_STRING = "Save";
    final static String NOTSAVE_STRING = "Don't Save";
    final static String CANCEL_STRING = "Cancel";
    final static String[] SAVE_BUTTONS = {SAVE_STRING, NOTSAVE_STRING, 
					  CANCEL_STRING};

    final static String YES_STRING = "Yes";
    final static String NO_STRING = "No";
    final static String[] YESNO_BUTTONS = {YES_STRING, NO_STRING};

    final static String QSAVE_STRING = "Save & Quit";
    final static String QUIT_STRING = "Quit";
    final static String[] QUIT_BUTTONS= {QSAVE_STRING, QUIT_STRING, 
					 CANCEL_STRING};

    final static String CONTINUE_STRING = "Continue";
    final static String SKIP_STRING = "Skip";
    final static String[] ARESAME_BUTTONS = {CONTINUE_STRING, SKIP_STRING,
					     CANCEL_STRING};

    /**
     * Allocates a new instance of the class ProdEnvironment.
     */  
    public ProdEnvironment()
    {
	super();
	setFont(Params.normalFont);	
	SetUpMenu();
	Save = new SaveText(this);
	setTitle(Params.Name + Save.Filename());
        setBackground(Params._pe_backcolor);

	//imported from JFLAP Grammar Input Window
	myNumLines = 10;
	myLeftFields = new Vector();
	myRightFields = new Vector();
    
	ScrollPane topScroll = new ScrollPane();
	topScroll.setSize(300, 300);
        topScroll.setBackground(Color.white);
	Panel buttonPanel = new Panel(new FlowLayout());
	add(topScroll, "Center");
	add(buttonPanel, "South");
    
	myScrollPanel = new Panel(new FlowLayout());
	Panel pan = new Panel(new BorderLayout());
	myScrollPanel.add(pan);
	myFieldPanel = new Panel(myFieldGrid = new GridLayout(10,1));
	myNumberPanel = new Panel(myNumberGrid = new GridLayout(10,1));
	pan.add(myFieldPanel, "Center");
	pan.add(myNumberPanel, "West");
	topScroll.add(myScrollPanel);
    
	Panel p;
	TextField t1, t2;
	for(int i=0; i< myNumLines; i++)
	{
	    p = new Panel(new FlowLayout());
	    t1 = new ModifyTextField(5);
	    t2 = new ModifyTextField(10);
	    myNumberPanel.add(new Label( (i+1) + ": "));
	    p.add(t1);
            p.add(new ArrowCanvas(20, 10));
	    p.add(t2);
	    myLeftFields.addElement(t1);
	    myRightFields.addElement(t2);
	    myFieldPanel.add(p);
	}
	((TextField) myLeftFields.elementAt(0)).setText("S");

	p = new Panel(new FlowLayout());
	button_1 = new Button("Parse");
	button_1.addActionListener(this);
	buttonPanel.add(button_1);

	button_2 = new Button("Transform Grammar");
	button_2.addActionListener(this);
	buttonPanel.add(button_2);

	addWindowListener(
	    new WindowAdapter()
	    {
		public void windowClosing(WindowEvent e)
		{
		    Quit();
		}
	    }
	);
    }

    private void SetUpMenu()
    {
	MenuBar mbar = new MenuBar();
	Menu m;
	Component c;
	Color color = Params.normalColor;
	m = new Menu("File");
	m.setFont(Params.menuFont);	
	MenuItem mi = new MenuItem("New Grammar");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mi = new MenuItem("Open Grammar");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	m.addSeparator();
	mi = new MenuItem("Save");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mi = new MenuItem("Save As...");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	m.addSeparator();
	mi = new MenuItem("Print");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	m.addSeparator();
	mi = new MenuItem("Quit");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mbar.add(m);
	
	m = new Menu("Options");
	m.setFont(Params.menuFont);
	m.add(mi = new MenuItem("Clear"));
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi = new MenuItem("Add Inputs"));
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	mbar.add(m);

	m = new Menu("Help");
	m.setFont(Params.menuFont);	
	mi = new MenuItem("Help HTML");
	mi.addActionListener(this);
	mi.setFont(Params.normalFont);
        m.add(mi);
	mi = new MenuItem("Help Text");
	mi.addActionListener(this);
	mi.setFont(Params.normalFont);
        m.add(mi);
	m.addSeparator();
	mi = new MenuItem("About...");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mbar.add(m);
	setMenuBar(mbar);	
    }	

    public void actionPerformed(ActionEvent e)
    {
	String arg;
	if (e.getSource() instanceof Button)
	{
	    arg = ((Button)e.getSource()).getLabel();
	    if (arg.equals("Transform Grammar")){
		RunProgram();
		return;
	    }
	    else if(arg.equals("Parse")){
		Parse();
		return;
	    }
	    else return;
	}
	else if (e.getSource() instanceof MenuItem)
	{	
	    arg = ((MenuItem)e.getSource()).getLabel();
	    if (arg.equals("Quit"))
	    {
	    	Quit();
	    	return;
	    }
	    else if (arg.equals("Save"))
	    {
		if(isModified())
		{
		    if(checkProductions())
		    {
			Save.save(getGrammar());
			removeModified();
		    }
		    else
		    {
			OKDialog dg = new OKDialog("File not saved.", this, "Warning...");
			dg.showResize();
		    }
		}
		setTitle(Params.Name + Save.Filename());
		return;
	    }	
	    else if (arg.equals("Save As..."))
	    {
		if(checkProductions())
		{
		    if(Save.saveAs(getGrammar()))
		    {
			setTitle(Params.Name + Save.Filename());
			removeModified();
		    }
		}
		else
		{
		    OKDialog dg = new OKDialog("File not saved.", this, "Warning...");
		    dg.showResize();
		}
		return;
	    }	
	    else if (arg.equals("Open Grammar"))
	    {
		OpenGrammar();
		setTitle(Params.Name + Save.Filename());
	    	return;
	    }
	    else if (arg.equals("New Grammar"))
	    {
		if(isModified())
		{
		    String pushedB;
		    TextInDialog dg = new TextInDialog("Productions are modified!\nDo you want a new grammar?", YESNO_BUTTONS, this, "New Grammar Warning!!");
		    dg.showResize();
		    pushedB = dg.getPushedButton();
		    if(pushedB.equals(YES_STRING))
			New();
		}
		else
		    New();
	    }
	    else if (arg.equals("Clear"))
	    {
		clear();
	    }
	    else if (arg.equals("Add Inputs"))
	    {
		addInputs();
	    }
	    else if(arg.equals("About..."))
	    {
		//Open up the about dialog
		AboutDialog ad = new AboutDialog(this, Params.trans2Pic, "",
				 Params.aboutHeight, Params.aboutWidth);
		ad.show();
		return;
	    }
	    else if(arg.equals("Help HTML"))
	    {
		//HelpDialog hd = new HelpDialog(0, this);
		//hd.show();
		HelpWindow help = HelpWindowHTML.getHelpWindow("PATE",
		    Params._hd_bkcolor);
		help.setVisible(true);
	    }
	    else if(arg.equals("Help Text"))
	    {
		//HelpDialog hd = new HelpDialog(0, this);
		//hd.show();
		HelpWindow help = HelpWindowText.getHelpWindow("PATE",
		    Params._hd_bkcolor);
		help.setVisible(true);
	    }
	    else
	    {
	    	NotYetMessage();
	    	return;
	    }	
	}
	return;
    }

    private void reset(GridBagConstraints con) {
	con.gridx = GridBagConstraints.RELATIVE;
	con.gridy = GridBagConstraints.RELATIVE;
	con.gridwidth = 1;
	con.gridheight = 1;
 
        con.weightx = 0;
	con.weighty = 0;
	con.anchor = GridBagConstraints.CENTER;
	con.fill = GridBagConstraints.NONE;
 
	con.insets = new Insets(0, 0, 0, 0);
	con.ipadx = 0;
	con.ipady = 0;
    }

    //Duhhhhh!!! You cant figure this one out!!??
    public void Quit()
    {
	if(isModified())
	{
	    String pushedB;
	    TextInDialog dg = new TextInDialog(
		"Grammar is modified and has\n" + 
		"not been saved!!!", QUIT_BUTTONS, 
		this, "Warning");
	    dg.showResize();
	    pushedB = dg.getPushedButton();
	    if(pushedB.equals(QSAVE_STRING))
	    {
		SaveText Save = new SaveText(this);

		if(Save.saveAs(getGrammar()))
		    System.exit(0);
		else
		    return;
	    }
	    else if(pushedB.equals(QUIT_STRING))
		System.exit(0);
	    else
		return;
	}
	else
	    System.exit(0);
    }
    
    public void Parse()
    {
        if(checkProductions())
	{
	    ParserEnvironment ParserWind = new ParserEnvironment(
		getGrammar(),Save.Filename());
	    ParserWind.setSize(360,520);
	    ParserWind.show();
	}

    }

    private boolean AlreadySaved()
    {
	String pushedB;
	if(isModified())
	{
	    TextInDialog dg = new TextInDialog("Productions have been altered\nand have not been saved.", SAVE_BUTTONS, this, "Warning");
	    dg.showResize();

	    pushedB = dg.getPushedButton();
	    if (pushedB.equals(SAVE_STRING)){
		if(checkProductions())
		{
		    return Save.save(getGrammar());
		}
		else
		    return false;
	    }
	    else if (pushedB.equals(CANCEL_STRING)){
		return false;
	    }
	    else if(pushedB.equals(NOTSAVE_STRING)){
		return true;
	    }
	}
	return true;
    }

    public boolean OpenGrammar()
    {
	if (AlreadySaved())
        {
	    ReadFile rf = new ReadFile();
	    FileDialog fd = new FileDialog(this, "Open ", FileDialog.LOAD);
//	    fd.setBackground(Params._dg_backcolor);
	    fd.setFont(Params.fileFont);
	    fd.show();
	    String File = fd.getFile();
	    String Dir = fd.getDirectory();
	    if (Dir == null || File == null)
		return false;
	    String fName = Dir + File;
	    if (!fName.endsWith(".GRM"))
		fName += ".GRM";
	    try
	    {
		rf.read(fName);
                clear();
		setGrammar(rf.g);
		Save = new SaveText(this, fName);
		removeModified();
		return true;
	    }
	    catch (FileNotFoundException e) {
		OKDialog dg = new OKDialog("Could not find file\n"+"'"+fName+"'", this, "Error");
		dg.showResize();
		return false;
	    }
	    catch (EOFException e) {
		OKDialog dg = new OKDialog("Unexpected end of file.", this, "Error");
		dg.showResize();
		return false;
	    }
	    catch (NumberFormatException e) {
		OKDialog dg = new OKDialog("Numbers not found where\nthey were needed...", this, "Error");
		dg.showResize();
		return false;
	    }
	    catch (IOException e) {
		OKDialog dg = new OKDialog("An I/O error occured.", this, "Error");
		dg.showResize();
		return false;
	    }
	    catch (Exception e) {
		OKDialog dg = new OKDialog("File corrupted " + e.getMessage(), this, "Error");
		dg.showResize();  
		return false;
	    }
	}
	return false;
    }

    //Makes a new grammar
    private void New()
    {
	//text_1.writeGrammar("S"+g.arrow,g);
	//text_1.update();
	clear();
	Save = new SaveText(this);
    }

    public void PrintGrammar()
    {
    }

    public void Index()
    {
    }

    /**
     * clear all input fields
     */
    protected void clear()
    {
	ModifyTextField tf;
	for(Enumeration e = myLeftFields.elements(); e.hasMoreElements(); ) {
	  tf = (ModifyTextField) e.nextElement();
	  tf.setText("");
	  tf.resetModifiedFlag();
	}
	for(Enumeration e = myRightFields.elements(); e.hasMoreElements(); ) 
	{
	  tf = (ModifyTextField) e.nextElement();
	  tf.setText("");
	  tf.resetModifiedFlag();
	}
	((TextField) myLeftFields.elementAt(0)).setText("S");
        ((ModifyTextField) myLeftFields.elementAt(0)).setModifiedFlag();
    }

    /**
     * add 4 rules to the list 
     */
    protected void addInputs()
    {
	Panel p;
	TextField t1, t2;
	
	for(int i = 0; i < 4; i++)
	{
	    p = new Panel(new FlowLayout());
	    t1 = new ModifyTextField(5);
	    t2 = new ModifyTextField(10);
	    myNumberPanel.add(new Label( (myNumLines+i+1) + ": "));
	    p.add(t1);
	    p.add(new ArrowCanvas(20, 10));
	    p.add(t2);
	    myLeftFields.addElement(t1);
	    myRightFields.addElement(t2);
	    myFieldPanel.add(p);
	}
	myNumLines +=4;
    
	myFieldGrid.setRows(myNumLines);
	myNumberGrid.setRows(myNumLines);
	pack();
    }

    public void NotYetMessage()
    {
	String m = new String("not yet");
	OKDialog NotYet = new OKDialog("This option is not yet implemented",
				       this, "Sorry!  :-(");
	NotYet.showResize();
    }

    private void RunProgram()
    {
	if(checkProductions())
	{
            if(getGrammar().IsUnrestricted())
            {
               OKDialog dlg = new OKDialog(
                  "Transformations cannot be performed\n" +
                  "on unrestricted grammars.",
                  this, "Sorry!  :-(");
               dlg.showResize();
               return;
            }
	    TransformWindow trans = new TransformWindow(
		getGrammar(),
		Save.Filename());
	    trans.addTransformProc(new TransLambda());
	    trans.addTransformProc(new TransUnit());
	    trans.addTransformProc(new TransUseless());
	    trans.addTransformProc(new TransCNF());
	    trans.nextTransformation();
	    trans.show();
	}
    }

    /**
     * Check whether any of the text fields have been modified
     */
    private boolean isModified()
    {
	for(int x = 0; x < myLeftFields.size(); ++x)
	{
	    if(((ModifyTextField)myLeftFields.elementAt(x)).modified)
	    {
		return true;
	    }
	    if(((ModifyTextField)myRightFields.elementAt(x)).modified)
	    {
		return true;
	    }
	}
	return false;
    }

    /**
     * removes the modification flags for all text fields
     */
    private void removeModified()
    {
	for(int x = 0; x < myLeftFields.size(); ++x)
	{
	    ((ModifyTextField)myLeftFields.elementAt(x)).resetModifiedFlag();
	    ((ModifyTextField)myRightFields.elementAt(x)).resetModifiedFlag();
	}
    }

    /**
     * Check whether any of the text fields have been modified
     */
    private Grammar getGrammar()
    {
	String temp;
	Grammar g = new Grammar();
	for(int x = 0; x < myLeftFields.size(); ++x)
	{
	    temp = "";
	    if(!((ModifyTextField)myLeftFields.elementAt(x)).getText().trim().equals(""))
	    {
		temp += ((ModifyTextField)myLeftFields.elementAt(x)).getText();
		temp += "->";
		if(!((ModifyTextField)myRightFields.elementAt(x)).getText().trim().equals(""))
		{
		    temp += ((ModifyTextField)myRightFields.elementAt(x)).getText();
		}
		else
		{
		    temp += Params.lambdaChar + "";
		}
		g.addProduction(temp);
	    }
	}
	return g;
    }

    /**
     * Sets the specified grammar into the text fields
     */
    private void setGrammar(Grammar g)
    {
	while(g.NumProductions() > myLeftFields.size())
	{
	    addInputs();
	}
	Enumeration e1 = myLeftFields.elements();
	Enumeration e2 = myRightFields.elements();
	for(g.FirstProd(); !g.ProdsDone(); g.NextProd())
	{
	    ((TextField) e1.nextElement()).setText(g.CurrentLeftProd());
            if(g.CurrentRightProd().equals("" + Params.lambdaChar))
               ((TextField) e2.nextElement()).setText("");
            else
	       ((TextField) e2.nextElement()).setText(g.CurrentRightProd());
	}
    }

    /**
     * Checks whether all the productions are in a valid
     * format
     */
    private boolean checkProductions()
    {
	boolean foundstart = false;
	for(Enumeration e = myLeftFields.elements(); e.hasMoreElements(); )
	{
	    if(((TextField)e.nextElement()).getText().trim().equals("S"))
	    {
		foundstart = true;
		break;
	    }
	}
	if(!foundstart)
	{
	    OKDialog dg = new OKDialog("No start production.",
		this, "Error");
	    dg.showResize();
	    return false;
	}

	for(int x = 0; x < myLeftFields.size(); ++x)
	{
	    if(((TextField)myLeftFields.elementAt(x)).getText().equals(""))
	    {
		if(!((TextField)myRightFields.elementAt(x)).getText().equals(""))
		{
		    OKDialog dg = new OKDialog("Cannot have lambda on the\n" +
			"right hand side.",
			this, "Error");
		    dg.showResize();
		    return false;
		}
	    }
	}
	return true;
    }
}



