package PateLib;

import java.util.*;
import java.awt.*;
import java.io.*;

/*
---------------------------------------------------------------------------
File	    :		SaveText.java
Package	    :		PateLib  Version 1.0
Author	    :		Jason Robert Salemme
Programmed  :		8 Aug 96	
Modified    :
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme
 * Computer Science Department
 * Duke University
 * August 1996
 * Supported by National Science Foundation DUE-9596002 and DUE-9555084.
 *
 * Copyright (c) 1996
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * This class saves the grammar, provided that the grammar
 * productions are syntatically correct.
 * 
 * @author Jason R. Salemme
 * @version 1.0, 08 Aug 96
 */
public class SaveText {

    private String myFilename;
    private Frame myParent;
    private final static String NO_FILE_NAME = "Untitled";

/**
 * Allocates a new instance of the class SaveText.  The filename if set to a 
 * default value.
 * @param parent the parent Frame of the grammar
 */
    public SaveText(Frame parent)
    {
	this.myParent = parent;
	this.myFilename = NO_FILE_NAME;
    }

/**
 * Allocates a new instance of the class SaveText with a specified filename.
 * @param parent the parent Frame of the grammar
 * @param fname the name of the file
 */
    public SaveText(Frame parent, String fname){
	this.myParent = parent;
	this.myFilename = fname;
    }

/**
 * Saves the grammar.  In Pate the grammar is obtained from the newTextArea so
 * make sure that the grammar is up-to-date.
 */
    public boolean save(Grammar g)
    //precond:  Text (grammar) must not be modified and should have already
    //		been checked.
    {
	if (NO_FILE_NAME.equals(myFilename))
	    return saveAs(g);
	else
	    return SaveGrammar(myFilename, g);
    }

/**
 * Saves the grammar with a new filename to be determined by the user in a 
 *file dialog.
 */
    public boolean saveAs(Grammar g)
    {
	FileDialog fd = new FileDialog(myParent,"Save Grammar" ,FileDialog.SAVE);
	fd.setFont(Params.fileFont);
	fd.setDirectory(".");
	fd.show();
	String File = fd.getFile();
	String Dir = fd.getDirectory();
	if (Dir == null || File == null){
	    return false;
	}
	String fName = Dir + File;
	return SaveGrammar(fName, g);
    }

    //Saves a file with the given fName    	
    private boolean SaveGrammar(String fName, Grammar g)
    {
	WriteFile wf = new WriteFile(g);
	try{
	    if (!fName.endsWith(".GRM"))
	       fName += "." + "GRM";
	    wf.write(fName);
	    myFilename = fName;
	    return true;
	}
	catch (IOException e) {
	    OKDialog dg = new OKDialog("I/O Error.\nFile not saved.\n"+e.getMessage(), myParent, "Error");
	    dg.showResize();
	    return false;
	}
	catch (NoSuchElementException e) {
	    OKDialog dg = new OKDialog("Tokenizer Error.\nFile not saved.\n"+e.getMessage(), myParent, "Error");
	    dg.showResize();
	    return false;
	}
	catch (Exception e) {
	    OKDialog dg = new OKDialog("File Error.\nFile not saved.\n"+e.getMessage(), myParent, "Error");
	    dg.showResize();
	    return false;
	}
    }	// end of method save(String).

    public String Filename(){
    //postcond:  returns the filename without the directory
	String name = new String();
	StringTokenizer st = new StringTokenizer(myFilename, "/");
	int num = st.countTokens();
	for(int k = 0; k < num; k++){
	    name = st.nextToken();
	}
	return name;
    }
}

