/*---------------------------------------------------------------------------
File:					Pate.java
Package:				PateLib  Version 1.0
Author:					Anna O. Bilska (Anya) V1.0 08/07/96
                                
Description of Contents:       
				
--------------------------------------------------------------------------*/

/* 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme
 * Computer Science Department
 * Duke University
 * August 1996
 * Supported by National Science Foundation DUE-9596002 and DUE-9555084.
 *
 * Copyright (c) 1996
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

// Anya Bilska 
// summmer 1996
// a class which constructs a scrollable canvas



// ***************************************************
// ***METHOD INDEX************************************
// ***************************************************
// CONSTRUCTORS:
// public ScrollCanvas(int x, int y)
// public ScrollCanvas()
// METHODS:
// public void setSize(int x, int y)
// public void processEvent(AWTEvent e)
// public void paint(Graphics g)
// public void update(Graphics g)
// private void DrawDerivation(Graphics g)
// public void DrawTree()
// public void DerivationToCanvas(Vector d, Vector p, Grammar g)
// public void DerivationToCanvas(TreeNode t)
// public void MessageToCanvas(String s)
// private void FindLongestLeft()


public class ScrollCanvas extends Panel implements AdjustmentListener
{
	public ScrollCanvas(int x, int y)
	// constructor 
	// x: width of the canvas to be
	// y: height of the canvas to be
	{
		super();
		myWidth = x;
		myHeight = y;
		mdy = 20;
		dx = 0;
		dy = 0;
		needsUpdating = false;
		myWords = new Vector();
		myProdsUsed = new Vector();
		shiftee = new Shifter();
		mesg = new String("");

		//System.out.println("hbar: " + myWidth);
		//System.out.println("vbar: " + myHeight);
		hbar = new Scrollbar(Scrollbar.HORIZONTAL,0,ScrollIncrement,0,0);
		hbar.addAdjustmentListener(this);
		vbar = new Scrollbar(Scrollbar.VERTICAL,0, ScrollIncrement, 0,0);
		vbar.addAdjustmentListener(this);
		setBackground(Params._scroll_bkcolor);
		vbar.setBackground(Params._scroll_barcolor);
		hbar.setBackground(Params._scroll_barcolor);
		setLayout(new BorderLayout());
		add("East", vbar);
		Panel p = new Panel(new BorderLayout());
		spacer = new Canvas();
		spacer.setEnabled(false);
		spacer.setBackground(Color.lightGray);
		p.add(hbar, "Center");
		p.add(spacer, "East");
		add(p, "South");
	}
	
	public ScrollCanvas()
	// constructor
	{
		this(1,1);
	}

	public void setSize(int x, int y)
	// postcondition: defines the size of the
	// canvas to be 'x' by 'y'
	{
	    myWidth = x;
	    myHeight = y;
	    d = this.getSize();
	    if (d.height<myHeight)
		vbar.setValues(0,ScrollIncrement,0,myHeight - d.height + vbar.getSize().width * 2);
	    else
		vbar.setValues(0,ScrollIncrement,0,0);
	    if (d.width<myWidth)
		hbar.setValues(0,ScrollIncrement,0,myWidth - d.width + hbar.getSize().height * 2);
	    else
		hbar.setValues(0,ScrollIncrement,0,0);
	}

	public void resizeScrollBars()
	{
	    spacer.setSize(vbar.getSize().width, vbar.getSize().width);
            paint(getGraphics());
	    validate();
	}
	
	public void adjustmentValueChanged(AdjustmentEvent e)
	// the event handling method
	{
	    if (e.getSource() == vbar)
	    {
		dy = vbar.getValue();
		repaint();
	    }
	    else if(e.getSource() == hbar) 
	    {
		dx = hbar.getValue();
		repaint();
	    }
	}

	public void paint(Graphics g)
	// the painting method
	{
           d = this.getSize();
           DrawFrame(g);
           if(needsUpdating || bufferedImage == null)
           {
               dx = dy = 0;
	       g.clearRect(0,0,d.width,d.height);

               redrawBufferedImage();

	       g.drawImage(bufferedImage,-dx,-dy,this);
	       DrawFrame(g);
	       needsUpdating = false;
           }
           else
           {
	       g.drawImage(bufferedImage,-dx,-dy,this);
	       DrawFrame(g);
           }
	}

        private void redrawBufferedImage()
        {
            myY = 20;
            if(bufferedImage != null)
	       bufferedImage.flush();
	    bufferedImage = createImage(2 * myWidth, 2 * myHeight);
	    Graphics g = bufferedImage.getGraphics();
            g.setFont(Params._scroll_drvfont);
	    DrawMessage(g);
	    
	    if(myTree == null)
	        DrawTextDerivation(g);
	    else
	        DrawTreeDerivation(g);
        }
	
	public void DrawMessage(Graphics g)
	// postcondition: draws the message on the screen
	{
		int x = 10;
		int dy = g.getFontMetrics().getHeight();
		String s;

		g.setColor(Params._scroll_highcolor);
		StringTokenizer st = new StringTokenizer(mesg,"\n");

		while(st.hasMoreTokens())
		{	
			s = st.nextToken();
			g.drawString(s,x,myY);
			myY += dy;
		}
	}

	public void DrawFrame(Graphics g)
	// postcondition: draws a frame at the
	// edges of the canvas
	{
		g.setColor(Color.gray);
		g.drawLine(0,0,d.width,0);
		g.drawLine(1,1,d.width-1,1);
		g.drawLine(0,0,0,d.height);
		g.drawLine(1,1,1,d.height-1);
	}

		
	public void update(Graphics g)
	{
		paint(g);
	}
	

	public void DerivationToCanvas(
           Vector d,
           Vector p,
           Grammar g)
	// postcondition: the canvas knows what to draw
	// (for textual output)
	{
		myWords = (Vector)d.clone();
		myProdsUsed = (Vector)p.clone();
		myGram = g;
		needsUpdating = true;
		FindLongestLeft();
                curRuleNum = 0;
		repaint();
	}

	public void DerivationToCanvas(
           Vector p,
           Grammar g,
           Vector t)
	// postcondition: the canvas knows what to draw
	// (for graphical output)
	{
		myTree = t;
		myGram = g;
                myProdsUsed = (Vector)p.clone();
		needsUpdating = true;
                curRuleNum = 0;
		repaint();
	}

	public void MessageToCanvas(String s)
	// sends a message to the canvas
	{
		mesg = s;
		needsUpdating = true;
		repaint();
	}

	private void FindLongestLeft()
	// finds the length of the longest left
	// line to be printed to the screen
	// (for textual output)
	{
		maxLength = 0;
		int len = myWords.size();
		for (int i=0; i<len; i++) 
		{
			if (maxLength<((String)myWords.elementAt(i)).length())
				maxLength = ((String)myWords.elementAt(i)).length();
		}
		maxLength *= 9;
		if (maxLength < 80)
			maxLength =80;
	}

        /**
         * Draws the text derivation to the canvas.
         */
	private void DrawTextDerivation(Graphics g)
	// postcondition: draws the text derivation
	// on the canvas
	{
           int temp, index;
           int len = myWords.size();
           int i,k;
           FontMetrics fm = g.getFontMetrics();
           mdy = g.getFontMetrics().getHeight(); //20
           myY += mdy;
           myY += mdy;
           mdy += 7;

           if(myWords.size() > 1)
           {
             g.setColor(Params._scroll_drvcolor);
             g.drawString("Derivation: ",10,myY);
             g.setColor(Params._scroll_prodcolor);
             g.drawString("Productions Used: ",maxLength +40,myY);
             myY += mdy;
             g.setColor(Params._scroll_drvcolor);
             g.drawString("S =>",10,myY);
             // output the first production used
             /*g.setColor(Params._scroll_prodcolor);
             temp=((Integer)myProdsUsed.elementAt(len-2)).intValue();
             k = shiftee.GetProd(temp);
             index = shiftee.GetIndex(temp);
             g.setColor(Params._scroll_prodcolor);
             g.drawString(myGram.LeftProdAt(k)+"->"+myGram.RightProdAt(k), 
                maxLength+40, myY+(mdy/2)); 
             myY += mdy;*/


             // output the rest
             //for (i=(len-2); i>0; i--)
             //for(i = len - 1; i > (len - 1 - curRuleNum); i--)
             for(i = 1; i <= curRuleNum; i++)
             {
                temp=((Integer)myProdsUsed.elementAt(myProdsUsed.size() - i - 1)).intValue();
                k = shiftee.GetProd(temp);
                g.setColor(Params._scroll_prodcolor);
                String rightProd = myGram.RightProdAt(k);
                if(rightProd.charAt(0) == Params.lambdaCharValue)
                   rightProd = String.valueOf(Params.lambdaChar);
                g.drawString(myGram.LeftProdAt(k) + "->" + rightProd, 
                   maxLength+40, myY+(mdy/2));

                myY += mdy;

                //set the colors for the last 
                if(i == myWords.size() - 1)
                  g.setColor(Params._scroll_highcolor);
                else
                  g.setColor(Params._scroll_drvcolor);
                g.drawString("=>"+(String)myWords.elementAt(myWords.size() - i - 1), 20, myY);
             }

                /*if(i < 0)
                {
                   temp = ((Integer)myProdsUsed.elementAt(i)).intValue();
                   k = shiftee.GetProd(temp);
                   g.setColor(Params._scroll_prodcolor);
                   String rightProd = myGram.RightProdAt(k);
                   if(rightProd.charAt(0) == Params.lambdaCharValue)
                      rightProd = String.valueOf(Params.lambdaChar);
                   g.drawString(myGram.LeftProdAt(k) + "->" + rightProd, 
                      maxLength+40, myY+(mdy/2));

                   myY += mdy;

                   g.setColor(Params._scroll_highcolor);
                   g.drawString("=>"+(String)myWords.firstElement(),20,myY);
                }*/
          }
          //myWords.removeAllElements();
          //myProdsUsed.removeAllElements();
	}

        /**
         * Draws the tree derivation to the canvas.
         */
	public void DrawTreeDerivation(Graphics g)
	// postcondition: draws the graphical tree representation
	// to the screen
	{
	    int i=0;
	    
	    FontMetrics fm = g.getFontMetrics();
	    int width;
	    int height = fm.getAscent();
            TreeNode curnode;
            TreeNode childnode;
	    
	    if(myTree != null)
	    {	
               int dyfrom;

               //paint groupings
               g.setColor(Params._scroll_groupcolor);
               for(Enumeration e = myTree.elements(); e.hasMoreElements(); )
               {
                  curnode = (TreeNode) e.nextElement();
                  if(curnode.getRuleNum() > curRuleNum)
                     continue;
                  curnode.drawGroupNode(g);
               }

               //paint transitions
               g.setColor(Params._scroll_edgescolor);
               for(Enumeration e = myTree.elements(); e.hasMoreElements(); )
               {
                  curnode = (TreeNode) e.nextElement();
                  for(Enumeration e2 = curnode.children.elements(); e2.hasMoreElements(); )
                  {
                     childnode = (TreeNode) e2.nextElement();
		     if(childnode.group != null || childnode.getRuleNum() > curRuleNum)
			 continue;
                     dyfrom = 0;
                     if(curnode.group != null)
                     {
                        dyfrom += 3;
                     }
                     g.drawLine(
                        curnode.coordinate.x,
                        curnode.coordinate.y + TreeNode.diameter / 2 + dyfrom,
                        childnode.coordinate.x,
                        childnode.coordinate.y - TreeNode.diameter / 2);
                  }
               }

               //paint nodes
               for(Enumeration e = myTree.elements(); e.hasMoreElements(); )
               {
                  curnode = (TreeNode) e.nextElement();
                  if(curnode.getRuleNum() > curRuleNum)
                     continue;
                  curnode.drawTreeNode(g, curRuleNum);
               }
	    }
	}

	private void DrawArrow(Graphics g, int x1, int y1,
					int x2, int y2)
	// postcondition: draws a line connecting 
	// the nodes in the tree
	{
           g.drawLine(x1,y1,x2,y2);
	}

        /**
         * forces the canvase to repaint everything
         * and redraw the tree or the text
         */
        public void repaintUpdate()
        {
           redrawBufferedImage();
           repaint();
        }

        /**
         * Increments the current rule and shows
         * more of the derivation.  Returns whether
         * there are any more levels
         */
        public boolean nextRule()
        {
            ++curRuleNum;
            int maxRuleNum = 0;
            int num;

            if(myTree == null)
            {
               if(curRuleNum < myWords.size() - 1)
               {
                  return true;
               }
               else
               {
                  maxRuleNum = myWords.size() - 1;
               }
            }
            else
            {
               for(Enumeration e = myTree.elements(); e.hasMoreElements(); )
               {
                  num = ((TreeNode) e.nextElement()).getRuleNum();
                  if(num > curRuleNum)
                     return true;
                  else if(num > maxRuleNum)
                  {
                     maxRuleNum = num;
                  }
               }
            }

            curRuleNum = maxRuleNum;
            return false;
        }

        /**
         * Decrements the current rule and shows
         * less of the derivation.  Returns whether
         * there is a lower level after this one
         */
        public boolean prevRule()
        {
            --curRuleNum;
            if(curRuleNum <= 0)
            {
               curRuleNum = 0;
               return false;
            }
            return true;
        }

        /**
         * returns a string for the current rule
         */
        public String getCurrentRule()
        {
           if(curRuleNum == 0 || curRuleNum >= myProdsUsed.size())
              return "";

           int temp = ((Integer)myProdsUsed.elementAt(myProdsUsed.size() - curRuleNum - 1)).intValue();
	   int k = shiftee.GetProd(temp);

           return myGram.LeftProdAt(k) +
               " -> " + 
               myGram.RightProdAt(k);
        }

        private boolean needsUpdating;
	private int myY = 0;
	private int mdy;
	private String myText;
	private Scrollbar vbar,hbar;
	private Graphics myG;
	private Image bufferedImage;
	private int dx, dy;
	private int myHeight, myWidth;
	private Vector myWords; 
	private Vector myProdsUsed;
	private String mesg;
	private Grammar myGram;
	private int maxLength=0;
	private int ScrollIncrement=10;
	private Shifter shiftee;
	private Vector myTree;
	private Dimension d;
	private Canvas spacer;
        private int curRuleNum;
}
