/*---------------------------------------------------------------------------
File:					Pate.java
Package:				PateLib  Version 1.0
Author:					Anna O. Bilska (Anya) V1.0 08/07/96
                                
--------------------------------------------------------------------------*/

/* 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme
 * Computer Science Department
 * Duke University
 * August 1996
 * Supported by National Science Foundation DUE-9596002 and DUE-9555084.
 *
 * Copyright (c) 1996
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//package PateLib;


package PateLib;

import java.awt.*;
import java.util.*;

// a class which draws continuosly
// to a canvas (as a thread) and can
// be used to indicate that something
// is working (in progress)

// ***************************************************
// ***METHOD INDEX************************************
// ***************************************************
// CONSTRUCTORS:
// public SmilyIndicator(Canvas c)
// METHODS:
// private synchronized void SetDimensions()
// private synchronized void SetUpColors()
// public synchronized void ClearPage(Graphics g)
// public void DrawPage()
// public void run()

public class SmilyIndicator extends Thread
{
	private boolean threadSuspended;
	private boolean threadStopped;

	public SmilyIndicator(Canvas c)
	// constructor
	{
		threadSuspended = false;
		threadStopped = false;
		tt = 1;
		myCanvas = c;
		myCanvas.setBackground(Params._smily_bkcolor);
	}
		
	private synchronized void SetDimensions()
	// postcondition: the class knows the size
	// of the canvas
	{
		d = myCanvas.getSize();
		ycenter = d.height/2;
		xcenter = d.width/2;
	}
	
	private synchronized void SetUpColors()
	// postcondition: the colors are set (shades
	// of blue) for drawing
	{
		myColors.removeAllElements();
		for (int i=0; i<120; i+=5)
		{		
			myColors.addElement(new Color(46,100+i,216));
		}
	}

	public synchronized void ClearPage(Graphics g)
	// postcondition: the canvas is cleared, and the 
	// frame is set up
	{
	    if(g != null)
	    {
		g.setColor(Params._smily_bkcolor);
		g.fillRect(0,0,d.width,d.height);		
		g.setColor(Color.gray);
		g.drawLine(0,0,d.width,0);
		g.drawLine(1,1,d.width-1,1);
		g.drawLine(0,0,0,d.height);
		g.drawLine(1,1,1,d.height-1);
		g.setColor(Color.white);
		g.drawLine(d.width,0,d.width,d.height);
		g.drawLine(d.width-1,1,d.width-1,d.height-1);
		g.drawLine(0,d.height,d.width,d.height);
		g.drawLine(1,d.height-1,d.width-1,d.height-1);
		g.setColor(Color.black);
		g.drawString("Parse",d.width/2-19,d.height/2+5);
	    }
	}

	
	public void DrawPage()
	// draws the picture
	{
	    Graphics g = myCanvas.getGraphics();
	    if(g != null)
	    {
		int c=0;
		r = 1;
		t = 1;
		oldx = r * (int)Math.cos(t) + xcenter;
		oldy = r * (int)Math.sin(t) + ycenter;

		ClearPage(g);
		SetDimensions();
		g.clipRect(2,2,d.width-4,d.height-4);
		do
		{
			temp = Math.cos(t);
			x = (new Double(r*temp)).intValue() + xcenter;
			temp = Math.sin(t);
			y = (new Double(r*temp)).intValue() + ycenter;
			t += tt;
			r += 1;
			if (c>=20)
				c = 0;
			g.setColor((Color)myColors.elementAt(c));
			g.drawLine(oldx,oldy,x,y);
			oldx = x;
			oldy = y;
			try{
				sleep(4);
			}
			catch(InterruptedException e){}
			c++;		
		}while (x<max);
		g.dispose();
	    }
	}


	public void run()
	// method executed when the thread is
	// start()ed
	{
		SetDimensions();
		max = d.width*2-10;
		SetUpColors();
		while(true)
		{
			DrawPage();			
			try{
			    sleep(4);
			    synchronized(this)
			    {
				while (threadSuspended)
				    wait();
				if(threadStopped)
				    return;
			    }
			}
			catch(InterruptedException e){}

			
			tt += 1;
			if (tt>110)
				tt = 1;
		}
		
	}

	//safe versions of suspend and resume
	public synchronized void safeSuspend()
	{
	    threadSuspended = true;
	}

	public synchronized void safeResume()
	{
	    threadSuspended = false;
	    notify();
	}

	public synchronized void safeStop()
	{
	    threadStopped = true;
	}
		
	private Canvas myCanvas;
	private int x1,x2,y1,y2;
	private int dx;
	private int dx2;
	private double theta;
	private int xcenter;
	private int ycenter;
	private int max;
	private int x,y,oldx,oldy,r;
	private double t;
	private double temp;
	private double tt;
	private Dimension d;
	private Vector myColors= new Vector();
}



