package PateLib;

import java.util.*;
import java.awt.*;

/*
---------------------------------------------------------------------------
File	    :		State.java
Package	    :		PateLib  Version 1.0
Author	    :		Jason Robert Salemme
Programmed  :		8 Aug 96	
Modified    :
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme
 * Computer Science Department
 * Duke University
 * August 1996
 * Supported by National Science Foundation DUE-9596002 and DUE-9555084.
 *
 * Copyright (c) 1996
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * This class is implements a state of a machine but is used here in the 
 * implementation of the GraphCanvas for the visualization of the grammars.
 *
 * @see PateLib.GraphCanvas
 * @author Jason R. Salemme
 * @version 1.0, 08 Aug 96
 */
public class State{

/**
 * Allocates a new instance of the class State with the specified name, if it
 * terminates, and the graphical point the it is located at.
 * @param name the name (usually the variable) of the state
 * @param terminator indicates if it terminates through terminals (or the derivation of terminals if desired)
 * @param p the point it is located at
 */
    public State(String name, boolean terminator, Point p){
	myName = name;
	myTerminator = terminator;
	myPoint = p;
	myTransitions = new Vector(0);	//vector of Points
    }

/**
 * Allocates a new instance of the class State with the specified name, if it
 * terminates, and the graphical point the it is located at.
 * @param name the name (usually the variable) of the state
 * @param terminator indicates if it terminates through terminals (or the derivation of terminals if desired)
 * @param x the x coordinate
 * @param y the y coordinate
 */
    public State(String name, boolean terminator, int x, int y){
	myName = name;
	myTerminator = terminator;
	myPoint = new Point(x,y);
	myTransitions = new Vector(0);
    }

/**
 * Changes the location of the state.
 * @param x the new x coordinate
 * @param y the new y coordinate
 */
    public void move(int x, int y){
	myPoint.move(x, y);
    }

/**
 * Returns the name of the state
 */
    public String Name(){
	return myName;
    }

/**
 * Indicates if the states terminates (defined by the constructor).
 */
    public boolean isTerminator(){
	return myTerminator;
    }

/**
 * Returns the point location of the state.
 */
    public Point getPoint(){
	return myPoint;
    }

/**
 * Returns a vector of the location of transitions of the state
 * @see PateLib.State.addTransition(Point p)
 */
    public Vector getTransitions(){
	return myTransitions;
    }

/**
 * Adds a transition to the state.  This is done by giving the point location
 * of the state that it leads to.
 * @param p the point of the transition state
 */
    public void addTransition(Point p){
	myTransitions.addElement(p);
    }

/**
 * Indicates if the states has a transition state located at a specified point.
 * @param p the transition state
 */
    public boolean hasTransition(Point p){
	return myTransitions.indexOf(p) != -1;
    }

/**
 * Sets the transitions of a state with a specified vector of points.
 * @param transitions the Vector of transition points.
 */
    public void setTransitions(Vector transitions){
    //precond: the vector is a vector of Points
	myTransitions = transitions;
    }

/**
 * Changes the termination behavior of the state.
 * @param b indicates if the states terminates or not.
 */
    public void Terminates(boolean b){
	myTerminator = b;
    }

    private String myName;
    private boolean myTerminator;
    private Point myPoint;
    private Vector myTransitions;   //Vector of points
}
