package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;

/*
---------------------------------------------------------------------------
File	    :		TextInDialog.java
Package	    :		PateLib  Version 1.0
Author	    :		Magda & Octavian Procopiuc
Programmed  :		15 Jul 96
Modified    :		08 Aug 96   -	Jason R. Salemme
                           Aug 99   -   Ted Hung
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme
 * Computer Science Department
 * Duke University
 * August 1996
 * Supported by National Science Foundation DUE-9596002 and DUE-9555084.
 *
 * Copyright (c) 1996
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * This class allows the user to open a modal dialog box for user input.  The
 * user is not able to continue with the rest of the program until a button
 * is pushed.  Whatever method or class that uses this it can determine which
 * button was pushed and operate accordingly.
 */
public class TextInDialog extends Dialog implements ActionListener
{

    protected String	message;
    protected int	width = Params._dg_width;
    protected int	height = Params._dg_height;
    private String[]	splitMessage;
    protected String[]	buttons;
    protected String	thePushedButton = "";
    protected Panel     buttonPanel;

/**
 * Allocates a new instance of the class TextInDialog.
 * @param message the message to display
 * @param buttons a String array of button names to display
 * @param parent the parent frame
 * @param title the title for the dialog box.
 */
    public TextInDialog(String message, String[] buttons, 
			Frame parent, String title) {
	super(parent, title, true);
	this.message = message;
	this.buttons = buttons;
        setFont(Params._dg_font);
	setBackground(Params._dg_backcolor);
	setLayout(new BorderLayout());
        buttonPanel = new Panel();
	Button b;
	for (int i=0; i < buttons.length; i++)
	{
	    buttonPanel.add(b = new Button(buttons[i]));
	    b.addActionListener(this);
	}
	add("South", buttonPanel);
	setSize(width, height);
    }

/**
 * Allocates a new instance of the class TextInDialog.  This is usefull for 
 * longer messages that can not be diplayed with the default constructor.
 * @param message the message to display
 * @param buttons a String array of button names to display
 * @param parent the parent frame
 * @param title the title for the dialog box.
 * @param h the height of the dialog box (in pixles)
 * @param w the width of the dialog box (in pixles)
 */
    public TextInDialog(String message, String[] buttons, Frame parent, 
			String title, int h, int w)
    {
	super(parent, title, true);
	this.message = message;
	this.buttons = buttons;
        setFont(Params._dg_font);
	setBackground(Params._dg_backcolor);
	setLayout(new BorderLayout());
        buttonPanel = new Panel();
	Button b;
	for (int i=0; i < buttons.length; i++)
	{
	    buttonPanel.add(b = new Button(buttons[i]));
	    b.addActionListener(this);
	}
	add("South", buttonPanel);
	setSize(w, h);
    }

    public void calcSize()
    {

	Dimension d = getSize();
	pack();
	String str;
        int height = 0;
	FontMetrics fm = getGraphics().getFontMetrics();
	StringTokenizer st = new StringTokenizer(message, "\n", false);
	int count = st.countTokens();
	while (st.hasMoreTokens())
	{
	    str = st.nextToken();
	    if(d.width < fm.stringWidth(str))
		d.width = fm.stringWidth(str) + 40;
            height += fm.getHeight();
	}

        if(height > d.height)
           d.height = height;
	setSize(d);
    }

    public void showResize()
    {
	calcSize();
        if(getParent().getLocation().x + (getParent().getSize().width - getSize().width)/2 >=
           Toolkit.getDefaultToolkit().getScreenSize().width - getSize().width ||
           getParent().getLocation().y + (getParent().getSize().height - getSize().height)/2 >=
           Toolkit.getDefaultToolkit().getScreenSize().height - getSize().height)
           setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - getSize().width)/2,
                       (Toolkit.getDefaultToolkit().getScreenSize().height - getSize().height)/2);
        else
           setLocation(getParent().getLocation().x + (getParent().getSize().width - getSize().width)/2,
		       getParent().getLocation().y + (getParent().getSize().height - getSize().height)/2);
        show();
    }

    public void showNonModal()
    {
	setModal(false);
	calcSize();
        show();
    }

/**
 * Returns the string name of the button pushed in the dialog box.
 */
    public String getPushedButton() {
	String str = thePushedButton;
	thePushedButton = "";
	return str;
    }

    public void actionPerformed(ActionEvent e)
    {
	if (e.getSource() instanceof Button)
	{
	    thePushedButton = ((Button) e.getSource()).getLabel();
	    setVisible(false);
	}
	return;
    }	    // end of method action.

    public void paint(Graphics g) {
	String str;
	g.setColor(Params._dg_forecolor);
	int	i = 0;
	Dimension d = getSize();
	StringTokenizer st = new StringTokenizer(message, "\n", false);
	int count = st.countTokens();
	FontMetrics fm = g.getFontMetrics();
	while (st.hasMoreTokens())
        {
           str = st.nextToken();
           g.drawString(
              str,
              d.width/2 - fm.stringWidth(str)/2,
              d.height/2 - fm.getHeight() * (count/2 - i++));
	}
    }
}	// end of class TextInDialog.
