package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
---------------------------------------------------------------------------
File	    :		TransCNF.java
Package	    :		PateLib  Version 1.0
Author	    :		Jason Robert Salemme
Programmed  :		8 Aug 96	
Modified    :           Ted Hung 8/99
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme, Ted Hung
 * Computer Science Department
 * Duke University
 * August 1999
 * Supported by National Science Foundation DUE-9596002, DUE-9555084,
 *  and DUE-9752583.
 *
 * Copyright (c) 1999
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * This is not a true class.  This is part of the main application for Pate
 * which transformes the grammar into CNF (Chomsky Normal Form).
 */
public class TransCNF extends TransformProc
{		
    public Grammar answer;
    private Grammar temp;   //another temp Grammar
    private Vector dProds;

    public TransCNF()
    {
	answer = new Grammar();
    }

    public void setGrammar(Grammar g)
    {
	GiveMessage();
	gram = (Grammar) g.clone();
	MakeCNF();
    }

    public Grammar getAnswer()
    {
	return (Grammar) answer.clone();
    }

    public boolean checkAnswer(Grammar curans)
    {
	return answer.equals(curans);
    }

    public void modifyGraphFrame(GraphFrame frame)
    {
    }

    public Vector getVariables()
    {
       return null;
    }

    public boolean checkVariables(String check)
    {
	return true;
    }

    public String getDescription()
    {
	return "CNF Conversion";
    }

    public int getShowFlags()
    {
	return SHOW_MODIFY;
    }

    public Vector getSectionLabels()
    {
	Vector vect = new Vector();
	vect.addElement("");
	vect.addElement("");
	vect.addElement("Step 1: Transform the Grammar to CNF form");
	vect.addElement("Step 2: Parse");
	return vect;
    }

    public String getType()
    {
	return "CNF";
    }

    public Grammar getStartGrammar()
    {
	Grammar ret = new Grammar();
	String temp;
	for(gram.FirstProd(); !gram.ProdsDone(); gram.NextProd())
	{
	    temp = gram.CurrentRightProd();
	    //No unit productions so I can do this
	    if(temp.length() == 1 &&
	       !Character.isUpperCase(temp.charAt(0)))
	    {
		ret.addProduction(gram.CurrentProd());
	    }
	    
	    if(temp.length() == 2 && 
	       Character.isUpperCase(temp.charAt(0)) && 
	       Character.isUpperCase(temp.charAt(1)) )
	    {
		ret.addProduction(gram.CurrentProd());
	    }
	}

	return ret;
    }

    public void GiveMessage()
    {
	Message msg = new Message(
	    "Certain conventions are used\n" + 
	    "in this program for creating\n" + 
	    "new productions in the conversion\n" + 
	    "to Chomsky Normal Form.\n" + 
	    "They are...\n" + 
	    "1) For terminals:  B(a) -> a\n" + 
	    "Where a represents any terminal.\n" + 
	    " \n" + 
	    "2) For expansion:  D(#) -> AB\n" + 
	    "Where # is any integer and\n" + 
	    " A and B are any two variables,\n" + 
	    "including B(a) and D(#).\n" +
            "Integers used must start at 0,\n" +
            "increase in increments of 1, and\n" +
            "be numbered lower for rules near\n" +
            "the top of the source grammar." +
	    " \n" + 
	    "You must use B(a) and D(#)!\n" + 
	    "See Help for more info.", wnd, 
	    "Something to ponder...", 450, 230);
        msg.setLocation(wnd.getLocation().x + wnd.getSize().width,
                   wnd.getLocation().y);
	msg.showNonModal();
    }

    private void MakeCNF()
    {
	//Part one
	MoveTerminals();    //Adds A->a prods to temp grammar
	MakeSubstitutes();  //Subtitute a to B(a)->a and put in temp
			    //then Substitutes for the term and adds the
			    //new production to the temp grammar

	//Part two
	MoveValids();	    //Moves all prods of A->a and n=2 into t grammar
	MakeValid();	    //Makes the rest into valid prods for CNF
    }

    private void MoveTerminals()
    {
	temp = new Grammar();
	for(gram.FirstProd(); !gram.ProdsDone(); gram.NextProd()){
	    //No unit productions so I can do this
	    if(gram.CurrentRightProd().length() == 1){
		temp.addProduction(gram.CurrentProd());
		gram.RemoveCurrentProd();
	    }
	}
    }

    private void MakeSubstitutes()
    {
	String s, t;

	for(gram.FirstProd(); !gram.ProdsDone(); gram.NextProd()){
	    s = new String(gram.CurrentRightProd());
	    for(int k = 0; k < s.length(); k++){
		t = "" + s.charAt(k);
		if(Character.isLowerCase(s.charAt(k)) || 
		   gram.IsOther(s.charAt(k))){
		    //directly below is done to check to see if there are the
		    //approprate terminal prods in the grammar and to see
		    //if there is only one prod for that variable because
		    //we do not want to add words to the grammar by altering
		    //it to use A when A->a or A->Aa (can really mess things up
		    if(!temp.IsAProd("B(" + t + ")" + temp.arrow + t)){
			//Make new variable
			temp.addProduction("B(" + t + ")" + temp.arrow + t);
		    }
		    //Sub for the terminal
		    s = Substitute(s, k);
		}
		//Skips to the index before the next varaible
		//(Skips to the last parenthesis if a B(n) variable)
		k = gram.getEnd(s, k) - 1;
	    }
	    //Add the new production
	    temp.addProduction(gram.CurrentLeftProd() + gram.arrow + s);
	}
//	temp.PrintProductions();    
    }

    private String Substitute(String s, int k)
    {
	String a, b, news, add;
	a = "" + s.charAt(k);
	int len = s.length();
	news = new String();
	add = new String();
	

	for(temp.FirstProd(); !temp.ProdsDone(); temp.NextProd()){
	    if(temp.CurrentRightProd().equals(a)){
		add = new String(temp.CurrentLeftProd());
	    }
	    //If new variable for CNF break out, otherwise search for one
	    if(add.length() > 1)
		break;
	}

	//Modify the string
	if (k == 0)
	    news = add + s.substring(1,len);
	else if (k < len)
	    news = s.substring(0,k) + add + s.substring(k+1, len);
	else
	    news = s.substring(0,k) + add;

	return new String(news);      //reassign s for MakeSubstitutes();
    }

    private void MoveValids()
    {
	for(temp.FirstProd(); !temp.ProdsDone(); temp.NextProd()){
	    if(temp.CurrentRightProd().length() <= 2 ||
	       IsTwoVars(temp.CurrentRightProd())){
		answer.addProduction(temp.CurrentProd());
		temp.RemoveCurrentProd();
	    }
	}
    }

    private boolean IsTwoVars(String s)
    //precond:	String is right side of the production
    //postcond: Returns true if the string contains only two variables
    {
	//Has to be a variable!!
	if(Character.isLowerCase(s.charAt(0)))
	    return false;
	int place = gram.getEnd(s, 0);
	//Has to be a variable and must have more!!
	if(Character.isLowerCase(s.charAt(0)) || place == s.length())
	    return false;
	return gram.getEnd(s, place) == s.length();
    }

    private void MakeValid()
    //postcond: Makes a production contain only two variables and creates
    //		new ones so that it is possible
    {
	String news;
	int index1, index2;
	int count = 0;
	dProds = new Vector();
	int len, k, place, N, nextVar;
	boolean firstTime, change; 
	String makeup = new String();

	for(temp.FirstProd(); !temp.ProdsDone(); temp.NextProd()){
	    firstTime = true;
	    news = new String(temp.CurrentRightProd());
	    len = news.length();
	    k = -1; //set to -1 so that index 0 will be first looked at
	    N = 2;  //Sets for second variable
	    makeup = new String();
	    index2 = FindIndex(news); //Gets index of second 2 last var
	    
	    //Use D prods if they are already there
	    while(k != index2){
		index1 = GetNIndex(news, k, N);
		place = IsDProd(news.substring(index1, len));
		if(place != -1){
		    news = new String(news.substring(0,index1) + "D(" + place +
				      ")");
		    len = news.length();
		    makeup = new String(news.substring(index1, len)); 
		    firstTime = false;
		    break;
		}
		k = index1;
		N = 1;
	    }

	    //Finish up the job to make the prods only two vars long
	    while(!IsTwoVars(news)){
		index2 = FindIndex(news); //Gets index of second 2 last var
		len = news.length();
		answer.addProduction("D(" + count + ")" + answer.arrow +
				news.substring(index2, len));

		//creates the set of made variables to put in dProds
		if(firstTime){
		    makeup = new String(news.substring(index2, len));
		    firstTime = false;
		}
		else{
		    nextVar = GetNIndex(news, index2, 1);
		    makeup = new String(news.substring(index2, nextVar) + 
					makeup);
		}
		dProds.addElement(makeup);
		//end of dProds addition

		news = new String(news.substring(0, index2) + "D(" + 
				  count + ")");
		count++;
	    }
	    answer.addProduction(temp.CurrentLeftProd() + answer.arrow + news);
	}
    }

    private int IsDProd(String s)
    //postcond:  returns the element index that is equal to s (-1 otherwise)
    {
	for(int k = 0; k < dProds.size(); k++){
	    if(s.equals((String)dProds.elementAt(k)))
		return k;
	}
	return -1;
    }

    private int GetNIndex(String s, int a, int b)
    //postcond: returns the index of the bth variable starting at a
    {
	int count = 0;
	for(int k = a+1; k < s.length(); k++){
	    if(Character.isUpperCase(s.charAt(k))){
		count++;
		if(count == b)
		    return k;
	    }
	}
	return count;
    }

    private int FindIndex(String s)
    //precond:   s must contain atleast two variables
    //postcond:  returns the index of the second to last var in string
    {
	int count = 0;
	int k = 0;

	for(k = s.length()-1; k >= 0; k--){
	    if(Character.isUpperCase(s.charAt(k))){
		count++;
		if(count == 2)
		    break;
	    }
	}
	return k;
    }

    private Grammar Recreate(Grammar x){
    //postcond: returns a grammar that is recreated out of CNF form

	SubDVars(x);
	SubBVars(x);
	return x;
    }

    private void SubBVars(Grammar x){
    //postcond: subs. all B(a) variables with the terminal a

	String left = new String();
	String right = new String();

	x.FirstProd();
	while(!x.ProdsDone()){
	    left = new String(x.CurrentLeftProd());
	    if(IsBVar(left)){
		right = new String(x.CurrentRightProd());
		x.RemoveCurrentProd();
		SubVar(x, left, right);
	    }
	    else{
		x.NextProd();
	    }	    
	}
    }

    private boolean IsBVar(String left){
    //postcond: returns true if variable is B(a)
	Grammar t = new Grammar();  //local use

	//return flase if length != 4 it can not be a B(a) var!
	if(left.length() != 4)
	    return false;

	//Check the whole thing (it fits a pattern)
	if(left.charAt(0) != 'B' || left.charAt(1) != '(' ||
		    !(Character.isLowerCase(left.charAt(2)) || 
		    t.IsOther(left.charAt(2))) ||
		    left.charAt(3) != ')'){
	    return false;
	}

	//If it gets here it is a B(a) variable
	return true;
    }

    private void SubDVars(Grammar x){
    //postcond: subs. all B(a) variables with the terminal a

	String left = new String();
	String right = new String();

	x.FirstProd();
	while(!x.ProdsDone()){
	    left = new String(x.CurrentLeftProd());
	    if(IsDVar(left)){
		right = new String(x.CurrentRightProd());
		x.RemoveCurrentProd();
		SubVar(x, left, right);
	    }
	    else{
		x.NextProd();
	    }
	}
    }

    private boolean IsDVar(String left){
    //postcond: returns true if the left is a D(n) variable (n == int)

	//must be 4 or more variables
	if(left.length() < 4)
	    return false;

	//See if it starts correctly
	if(left.charAt(0) != 'D' || left.charAt(1) != '(')
	    return false;

	//Check the rest
	for(int k = 2; k < left.length(); k++){
	    if(!Character.isDigit(left.charAt(k))){
		//Check to see if it ends correctly
		if(left.charAt(k) != ')' && k != left.length()-1)
		    return false;
	    }
	}
	
	//If it gets to this point
	return true;
    }

    private void SubVar(Grammar x2, String left, String right){
    //postcond:  Subs all variables (left) as string right

	int ll = left.length();
	boolean changed;	//bool for change of word
	int index = -1;
	String word = new String();

	for(x2.FirstProd(); !x2.ProdsDone(); x2.NextProd()){
	    changed = false;
	    word = x2.CurrentRightProd();
    	    do{
		int wl = word.length();
		index = Contains(word, left);
		if(index != -1){
		    changed = true;
		    if(index == 0){
			 word = right + word.substring(index+ll, wl);
		    }
		    else if(index == wl-ll){
			word = word.substring(0,index) + right;
		    }
		    else{
			word = word.substring(0,index) + right + 
			    word.substring(index+ll, wl);
		    }
		}
	    }while(index != -1);
	    if(changed){
		x2.addProduction(x2.CurrentLeftProd() + x2.arrow + word);
		x2.RemoveCurrentProd();
		changed = false;
	    }
	}
	x2.FirstProd();
    }

    private int Contains(String word, String sub){
    //postcond: returns index to start of substring sub; -1 otherwise

	int wl = word.length();
	int sl = sub.length();

	if(wl < sl)
	    return -1;

	for(int k = 0; k <= wl - sl; k++){
	    //return index if contains the substring
	    if(sub.equals(word.substring(k, k+sl)))
		return k;
	}

	return -1;
    }

    private boolean ReconError(Grammar check, Grammar source){
    //This function checks to see which productions are not recreated
    //And tells the user what they are for their own debuggin purposes.
    
	Vector errors = new Vector();

	for(source.FirstProd(); !source.ProdsDone(); source.NextProd()){
	    if(!check.IsAProd(source.CurrentProd())){
		//Print the production
		errors.addElement(source.CurrentProd());
	    }
	}
	if(errors.size() > 0){
	    int wide = 230;
	    int height = 160;
	    int w = 0;

	    String e = "The following original\nproductions can not be recreated\nfrom your set of productions:\n \n";
	    for(int k = 0; k < errors.size(); k++){
		e = e + (String)errors.elementAt(k) + "\n";
		height += 20;
		w = ((String)errors.elementAt(k)).length()*7;
		if(wide < w)
		    wide = w;
	    }
	    Message m = new Message(e, wnd, "You have problems...", 
				    height, wide);
	    m.show();
	    return false;
	}
	return true;
    }
}




