package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
---------------------------------------------------------------------------
File	    :		TransLambda.java
Package	    :		PateLib  Version 1.0
Author	    :		Jason Robert Salemme
Programmed  :		8 Aug 96	
Modified    :           Ted Hung 8/99
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme, Ted Hung
 * Computer Science Department
 * Duke University
 * August 1999
 * Supported by National Science Foundation DUE-9596002, DUE-9555084,
 *  and DUE-9752583.
 *
 * Copyright (c) 1999
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */


/**
 * This is not a true class.  This is part of the main application for Pate
 * which removes all the lambda productions of a grammar.
 */
public class TransLambda extends TransformProc 
{
    public Grammar answer;
    public Grammar t;
    public Vector Vn;

    public TransLambda()
    {
	answer = new Grammar();
	Vn = new Vector();
    }

    public void setGrammar(Grammar g)
    {
	gram = (Grammar) g.clone();
	removeLambdas();
    }

    public Grammar getAnswer()
    {
	return (Grammar) answer.clone();
    }

    public boolean checkAnswer(Grammar curans)
    {
	return answer.equals(curans);
    }

    public void modifyGraphFrame(GraphFrame frame)
    {
    }

    public Vector getVariables()
    {
       return Vn;
    }

    public boolean checkVariables(String check)
    {
	Vector wrong = new Vector();
        Vector correct = new Vector();
        Vector repeated = new Vector();
	StringTokenizer t = new StringTokenizer(check, ", \n");
	String word;

	while(t.hasMoreTokens())
	{
	    word = t.nextToken();
	    word = word.trim();	    //remove white space
            if(Vn.indexOf(word) == -1)
	       wrong.addElement(word);
            else if(correct.contains(word))
               repeated.addElement(word);
            else
               correct.addElement(word);
	}
	
	if(wrong.size() > 0)
	{
	    //Print wrongs and give message
	    String p = new String();
	    p = makeString(wrong);
	    OKDialog dg = new OKDialog("The variable(s):\n"+p+"\nhave no lambda productions\nor do not exist!!!", wnd, "Invalid Variables");
	    dg.showResize();
	    return false;
	}
        else if(repeated.size() > 0)
        {
            //Print repeated and give message
	    String p = new String();
	    p = makeString(repeated);
	    OKDialog dg = new OKDialog("The variable(s):\n"+p+"\nare repeated!!!", wnd, "Invalid Variables");
	    dg.showResize();
	    return false;
        }
	else if(correct.size() != Vn.size())
        {
	    OKDialog dg = new OKDialog("You have not given all\nof the variables that\nhave lambda productions!", wnd, "Invalid Variables");
	    dg.showResize();
	    return false;
	}
	else
        {
	    //Give good job
	    OKDialog dg = new OKDialog("You got them all right!", wnd, "Correct Answers");
	    dg.showResize();
	    return true;
	}
    }

    public String getDescription()
    {
	return "Lambda Removal";
    }

    public int getShowFlags()
    {
	return SHOW_VARIABLE | SHOW_MODIFY;
    }

    public Vector getSectionLabels()
    {
	Vector vect = new Vector();
	vect.addElement("Step 1: Find the variables that derive lambda");
	vect.addElement("");
	vect.addElement("Step 2: Modify the grammar to remove the lambdas");
	vect.addElement("Step 3: Continue");
	return vect;
    }

    public String getType()
    {
	return "lambda";
    }

    public Grammar getStartGrammar()
    {
	return (Grammar) gram.clone();
    }

    private boolean IsNullable(String s)
    //Returns true if the production is nullable with lambda variables
    {
	String var = new String();

	for (int k = 0; k < s.length(); k++){
	    if (Character.isLowerCase(s.charAt(k)))
		return false;
	    else{
		var = s.substring(k, gram.getEnd(s, k));
		if (!IsLambdaVar(var))
		    return false;
	    }
	}
	return true;
    }
	
    private boolean IsALambda(String s)
    //returns true if the production is a lambda production
    {
	if (s.length() == 1 && s.charAt(0) == Params.lambdaChar)
	{
	    return true;
	}
	return false;
    }

    private boolean IsLambdaVar(String var)
    //Returns true if the variable is a lambda or nullable variable
    {
	return Vn.indexOf(var) != -1;
    }

    private int NumNulls(String s, Vector v)
    //returns the number of nullable variables within the string
    {
	int count = 0;
	int end;

	for (int k = 0; k < s.length(); k++){
	    end = gram.getEnd(s, k);
	    if (IsLambdaVar(s.substring(k, end)))
	    {
		count++;
	    }
	}
	return count;
    }

    private void FindNulls()
    {
	boolean change;
	t = (Grammar)gram.clone();

	//Remove all lambda productions and store them
	for(t.FirstProd(); !t.ProdsDone(); t.NextProd())
	{
	    if(IsALambda(t.CurrentRightProd()))
	    {
		if(Vn.indexOf(t.CurrentLeftProd()) == -1)
		{
		    Vn.addElement(t.CurrentLeftProd());
    		}
		t.RemoveCurrentProd();
	    }
	}
    
	answer = (Grammar)t.clone();    //removes the lambda productions
	
	Grammar t2 = (Grammar)t.clone();    //second local temp grammar for
					    //this next method

	//There has to be a better way than this but this will do.
	do{
	    change = false;
	    for(t2.FirstProd(); !t2.ProdsDone(); t2.NextProd()){
		if(IsNullable(t2.CurrentRightProd())){
		    if(Vn.indexOf(t2.CurrentLeftProd()) == -1){
			Vn.addElement(t2.CurrentLeftProd());
		    }
		    t2.RemoveCurrentProd();
		    change = true;
		}
	    }
	}
	while(change);
    }
    
    private int NextNull(String s, int j)
    //Precondition: s must contain at least j # of lambda (nullable) variable
    {
	int count = 0;
	int end;

	for (int k = 0; k < s.length(); k++){
	    end = gram.getEnd(s, k);
	    if (IsLambdaVar(s.substring(k, end)))
		if (count == j)
		    return k;
		else
		    count++;
	}
	return 0;		//this statement in so it will compile
    }

    private void Add(String s)
    {
	if(!answer.IsAProd(s)){
	    answer.addProduction(s);
	}
    }

    private boolean IsOneVar(String s){
    //precond:	s must be the right side of the production.
    //postcond:	returns true if s has only one var and no terminals
    //	    this method takes into account the B(a) and D(#) vars.
	return gram.getEnd(s, 0) == s.length();
    }

    private void Combo(int k, String left, String s)
    {
	if (k==0)
	    return;
	else{
	    int p, q;
	    for (int j = 0; j < k; j++){
		p = NextNull(s,j);  //beginnning index of null var
		q = gram.getEnd(s,p);   //1 + ending index of null var
		if (!IsOneVar(s)){	//do if it has more than one variable
		    if (p==0){
			Add(left + gram.arrow + s.substring(q,s.length()));
			Combo(k-1, left, s.substring(q,s.length()));
		    }
		    else if (p == s.length()){
			Add(left + gram.arrow + s.substring(0,p));
		    }
		    else{
			Add(left + gram.arrow + s.substring(0,p) + 
					 s.substring(q, s.length()));
			Combo(k-1, left, s.substring(0,p) + 
					 s.substring(q, s.length()));
		    }
		}
		else break;
	    }
	}
    }

    private void MakeCombos()
    {
	int N;
	for (t.FirstProd(); !t.ProdsDone(); t.NextProd()){
	    N = NumNulls(t.CurrentRightProd(),Vn);
	    Combo(N, t.CurrentLeftProd(), t.CurrentRightProd());
	}
    }

    private void removeLambdas()
    {
	FindNulls();
	MakeCombos();
    }
}


