package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
---------------------------------------------------------------------------
File	    :		TransUnit.java
Package	    :		PateLib  Version 1.0
Author	    :		Jason Robert Salemme
Programmed  :		8 Aug 96	
Modified    :           Ted Hung 8/99
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme, Ted Hung
 * Computer Science Department
 * Duke University
 * August 1999
 * Supported by National Science Foundation DUE-9596002, DUE-9555084,
 *  and DUE-9752583.
 *
 * Copyright (c) 1999
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */


public class TransUnit extends TransformProc 
{		
    private Grammar answer;
    private Grammar temp;
    private Hashtable graph;

    public TransUnit()
    {
	answer = new Grammar();
    }

    public void setGrammar(Grammar g)
    {
	gram = (Grammar) g.clone();
	removeUnitProds();
    }

    public Grammar getAnswer()
    {
	return (Grammar) answer.clone();
    }
    
    public boolean checkAnswer(Grammar curans)
    {
	return answer.equals(curans);
    }

    public void modifyGraphFrame(GraphFrame frame)
    {
	frame.setParameters(gram, false, new Vector());
	frame.setTitle("Production Visualization");
	frame.setSize(400, 400);
    }

    public Vector getVariables()
    {
       return null;
    }

    public boolean checkVariables(String check)
    {
	return true;
    }

    public String getDescription()
    {
	return "Unit Production Removal";
    }

    public int getShowFlags()
    {
	return SHOW_VISUALIZE | SHOW_MODIFY;
    }

    public Vector getSectionLabels()
    {
	Vector vect = new Vector();
	vect.addElement("");
	vect.addElement("Step 1: Visualize the unit productions");
	vect.addElement("Step 2: Remove the unit productions");
	vect.addElement("Step 3: Continue");
	return vect;
    }

    public String getType()
    {
	return "unit";
    }

    public Grammar getStartGrammar()
    {
	return (Grammar) gram.clone();
    }

    private boolean IsOneVar(String r)
    {
    //precond:	r must be the right side of the production
    //postcond: returns true if right side of prod is one var
	return Character.isUpperCase(r.charAt(0)) && 
	    gram.getEnd(r, 0) == r.length();
    }

    private void removeUnitProds()
    {
	temp = new Grammar();
	for(gram.FirstProd(); !gram.ProdsDone(); gram.NextProd())
	{
	    if(IsOneVar(gram.CurrentRightProd()))
		temp.addProduction(gram.CurrentProd());
	    else
		answer.addProduction(gram.CurrentProd());
	}
	FindDerivatives();
    }

    private void FindDerivatives()
    {
	graph = new Hashtable(151);

	for(temp.FirstProd(); !temp.ProdsDone(); temp.NextProd()){
	    if(!InGraph(temp.CurrentLeftProd())){
		graph.put(temp.CurrentLeftProd(), new Vector());
	    }
	    AppendUnit(new String(temp.CurrentLeftProd()),
		       new String(temp.CurrentRightProd()));
	}

	//Visit all the elements in the graph  to find derivatives
	Enumeration e = graph.keys();
	while(e.hasMoreElements()){	    
	    Vector seen = new Vector();
	    String s = (String)e.nextElement();	    
	    Vector prods = (Vector)graph.get(s);
	    AddDerivatives(prods, seen, s);
	}
//	System.out.println("----------");
//	temp.PrintProductions();

	//Make all substitutions into the grammar
	String s, tl, tr;
	Grammar temp2 = new Grammar();

	for(temp.FirstProd(); !temp.ProdsDone(); temp.NextProd()){
	    tr = temp.CurrentRightProd();
	    tl = temp.CurrentLeftProd();
	    for(answer.FirstProd(); !answer.ProdsDone(); answer.NextProd()){
		s = tl + answer.arrow + answer.CurrentRightProd();
		if(answer.CurrentLeftProd().equals(tr) &&
				!answer.IsAProd(s) &&
				!temp2.IsAProd(s))
		    temp2.addProduction(s);
	    }
	}
	//Put temp2 into answer for completion of transformation
	for(temp2.FirstProd(); !temp2.ProdsDone(); temp2.NextProd())
	{
	    answer.addProduction(temp2.CurrentProd());
	}	
    }

    private void AppendUnit(String l, String r)
    {
	Vector v = (Vector)graph.get(l);
	v.addElement(r);
    }

    private void AddDerivatives(Vector prods, Vector seen, String a)
    {
	String s;

	for(int k = 0; k < prods.size(); k++){
	    s = (String)prods.elementAt(k);
	    if(NotSeen(s, seen)){
		if(!temp.IsAProd(a + temp.arrow + s) && !a.equals(s)){
		    temp.addProduction(a + temp.arrow + s);
		}
		if(IsLeftVar(s)){
		    AddDerivatives((Vector)graph.get(s), seen, a);
		}
	    }
	}
    }
    
    private boolean IsLeftVar(String s)
    //precond:	s must be right side of the production
    //postcond:	returns true if there is a producitons that starts
    //		with s.
    {
	//check for productions in the unit grammer
	for(temp.FirstProd(); !temp.ProdsDone(); temp.NextProd()){
	    if (s.equals(temp.CurrentLeftProd()))
		return true;
	}
	return false;
    }

    private boolean NotSeen(String s, Vector v)
    {
	if(v.indexOf(s) != -1)
	    return false;
	v.addElement(s);
	return true;
    }

    private boolean InGraph(String s)
    {
	return graph.get(s) != null;
    }    
}
