package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
---------------------------------------------------------------------------
File	    :		TransUseless.java
Package	    :		PateLib  Version 1.0
Author	    :		Jason Robert Salemme
Programmed  :		8 Aug 96	
Modified    :           Ted Hung 8/99
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme, Ted Hung
 * Computer Science Department
 * Duke University
 * August 1999
 * Supported by National Science Foundation DUE-9596002, DUE-9555084,
 *  and DUE-9752583.
 *
 * Copyright (c) 1999
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */


/**
 * This is not a true class.  This is part of the main application for Pate
 * which removes the useless productions of a grammar.
 */
public class TransUseless extends TransformProc
{		
    private Grammar answer;
    private Vector Vt;	    //dump vector for terminating variables
    private Vector Vn;
    private Hashtable graph;	//Hashtable that represents the graph

    public TransUseless()
    {
	answer = new Grammar();
	Vt = new Vector();
	Vn = new Vector();
    }

    public void setGrammar(Grammar g)
    {
	gram = (Grammar) g.clone();
	removeUselessProds();
    }

    public Grammar getAnswer()
    {
	return (Grammar) answer.clone();
    }
    
    public boolean checkAnswer(Grammar curans)
    {
	return answer.equals(curans);
    }

    public void modifyGraphFrame(GraphFrame frame)
    {
	frame.setParameters(gram, true, Vn);
	frame.setTitle("Production Visualization");
	frame.setSize(400, 400);
    }

    public Vector getVariables()
    {
       return Vt;
    }

    public boolean checkVariables(String check)
    {
	Vector wrong = new Vector();
        Vector correct = new Vector();
        Vector repeated = new Vector();
	StringTokenizer t = new StringTokenizer(check, ", \n");
	String word;

	while(t.hasMoreTokens())
	{
	    word = t.nextToken();
	    word = word.trim();	    //remove white space
            if(Vt.indexOf(word) == -1 || correct.contains(word))
	       wrong.addElement(word);
            else if(correct.contains(word))
               repeated.addElement(word);
            else
               correct.addElement(word);
	}
	
	if(wrong.size() > 0)
	{
	    //Print wrongs and give message
	    String p = new String();
	    p = makeString(wrong);
	    OKDialog dg = new OKDialog("The variable(s):\n"+p+"\ndo not derive terminals\nor do not exist!!!", wnd, "Invalid Variables");
	    dg.showResize();
	    return false;
	}
        else if(repeated.size() > 0)
        {
            //Print repeated and give message
	    String p = new String();
	    p = makeString(repeated);
	    OKDialog dg = new OKDialog("The variable(s):\n"+p+"\nare repeated!!!", wnd, "Invalid Variables");
	    dg.showResize();
	    return false;
        }
	else if(correct.size() != Vt.size()){
	    OKDialog dg = new OKDialog("You have not given all\nof the variables that\nhave or derive terminals!", wnd, "Invalid Variables");
	    dg.showResize();
	    return false;
	}
	else
        {
	    //Give good job
	    OKDialog dg = new OKDialog("You got them all right!", wnd, "Correct Answers");
	    dg.showResize();
	    return true;
	}
    }

    public String getDescription()
    {
	return "Useless Production Removal";
    }

    public int getShowFlags()
    {
	return SHOW_VARIABLE | SHOW_VISUALIZE | SHOW_MODIFY;
    }

    public Vector getSectionLabels()
    {
	Vector vect = new Vector();
	vect.addElement("Step 1: Find the variables that derive terminals");
	vect.addElement("Step 2: Visualize the dependency graph");
	vect.addElement("Step 3: Modify the grammar to remove the useless productions");
	vect.addElement("Step 4: Continue");
	return vect;
    }

    public String getType()
    {
	return "useless";
    }

    public Grammar getStartGrammar()
    {
	return (Grammar) gram.clone();
    }

    private void removeUselessProds()
    {
	//Removing Non-Terminating Productions
	MakeGraph(gram);
	FindTerminators();
	FindDerivesTerminals();
	DetermineNonDerives();
	RemoveNon(Vn, gram);

	answer = (Grammar)gram.clone();

	//Removing Productions that cannot be reached by start var
	MakeGraph(answer);
	Vector visits = new Vector();
	VisitGraph(visits, answer.getStartSymbol());
	RemoveNonVisited(visits);
    }

    private void MakeGraph(Grammar t)
    {
	graph = new Hashtable(151);

	for(t.FirstProd(); !t.ProdsDone(); t.NextProd())
	{
	    if(!InGraph(t.CurrentLeftProd(), graph))
	    {
		graph.put(t.CurrentLeftProd(), new Vector());
	    }
	    AppendUnit(new String(t.CurrentLeftProd()),
		       new String(t.CurrentRightProd()));
	}
    }

    private void AppendUnit(String l, String r)
    {
	String s;
	char c;
	Vector v;
	int skipTo;
	v = (Vector)graph.get(l);

	for(int k = 0; k < r.length(); k++){
	    c = r.charAt(k);
	    if(Character.isUpperCase(c)){
		skipTo = gram.getEnd(r, k);
		s = r.substring(k, skipTo);
		if(v.indexOf(s) == -1)
		    v.addElement(s);
		k = skipTo - 1;
	    }
	}
    }

    private boolean InGraph(String s, Hashtable graph)
    //postcon: returns true is the variable is not in the graph
    //         as the left side of a production	
    {
	return graph.get(s) != null;
    }

    private void FindTerminators()
    //postcond: find all the vars that have prods that directly terminate
    {
	Vt = new Vector();
	for(gram.FirstProd(); !gram.ProdsDone(); gram.NextProd()){
	    if(Vt.indexOf(gram.CurrentLeftProd()) == -1 &&
		allLower(gram.CurrentRightProd())){
		    Vt.addElement(gram.CurrentLeftProd());
	    }
	}
    }

    private boolean allLower(String s)
    //postcond: returns true if all of the string is in lower case or other
    //		special terminal characters
    {
	for(int k = 0; k < s.length(); k++){
	    if(!Character.isLowerCase(s.charAt(k)) && !gram.IsOther(s.charAt(k)))
		return false;
	}
	return true;
    }

    private void FindDerivesTerminals()
    {
	//restart if a new variable derives terminals
	boolean restart = false;    

	for(gram.FirstProd(); !gram.ProdsDone(); gram.NextProd()){
	    if(restart){
		gram.FirstProd();
		restart = false;
	    }
	    restart = DeriveTerms(gram.CurrentLeftProd(),
		gram.CurrentRightProd());
	}
    }

    private boolean IsOnlyProd(String s)
    //precond:  The grammar must contain at least one production of var s
    //postcond: Returns true if variable s has only one production
    {
	Grammar z = (Grammar)gram.clone();
	int count = 0;
	for(z.FirstProd(); !z.ProdsDone(); z.NextProd()){
	    if(s.equals(z.CurrentLeftProd())){
		count++;
		if(count > 1)
		    return false;
	    }
	}
	return true;
    }

    private boolean DeriveTerms(String left, String right)
    //postcond: returns true if a new variable is found that derives terminals
    //This method goes through the graph determining if the right side of the
    //production derives terminals.
    {
	String t;
	Vector visits = new Vector();
	int skipTo;
	//keep track if a new derives terminal variable was found
	boolean addToVt = false;

	for(int k = 0; k < right.length(); k++){
	    if(Character.isUpperCase(right.charAt(k))){
		//Get variable - even if B(a) or D(#)
		skipTo = gram.getEnd(right, k);
		t = right.substring(k, skipTo);
		if(Vt.indexOf(t) == -1){
		    //cannot assume that the variable already terminates
		    if(t.equals(left))
			return false;
		    if (!CanReachTerminator(left, t, visits)){
			return false;
		    }
		    visits = new Vector();
		}
		k = skipTo - 1;
	    }
	}
	if(Vt.indexOf(left) == -1){
	    Vt.addElement(left);
	    addToVt = true;
	}

	return addToVt;
    }

    private boolean CanReachTerminator(String left, String var, Vector visits)
    //Finds the vector for variable t and searches through the graph
    {
	Vector v = (Vector)graph.get(var);
	if(v != null){
	    visits.addElement(var);
	    if(Terminates(v, left, visits))
		return true;
	}
	return false;		
    }

    private boolean Terminates(Vector v, String left, Vector visits)
    //The search through the vector part of the graph
    {
	String var;
	for(int k = 0; k < v.size(); k++){
	    var = (String)v.elementAt(k);
	    if(!var.equals(left)){
		if (Vt.indexOf(var) != -1){
		    return true;
		}
		else{
		    if(visits.indexOf(var) == -1 && 
		       CanReachTerminator(left, var, visits)){
			return true;
		    }
		}
	    }
	}
	return false;
    }

    private void DetermineNonDerives()
    //Makes a vector of the variables that do not derive terminals
    {
	Vector vars = gram.getVariables();
	for(int k = 0; k < vars.size(); k++){
	    if(Vt.indexOf(vars.elementAt(k)) == -1)
		Vn.addElement(vars.elementAt(k));
	}
    }

    private void RemoveNon(Vector v, Grammar p)
    //postcond:	removes any productions that have varialbes that do not 
    //		terminate (stored in Vt)
    {
	for(p.FirstProd(); !p.ProdsDone(); p.NextProd()){
	    if(HasNonTerm(v, p.CurrentProd())){
		p.RemoveCurrentProd();
	    }
	}
    }

    private boolean HasNonTerm(Vector v, String s)
    //postcond:	returns true if the production has a non terminating variable
    //		(terminating variables are in the vector VT).
    {
	String a;
	int skipTo;

	for(int k = 0; k < s.length(); k++){
	    if(Character.isUpperCase(s.charAt(k))){
		skipTo = gram.getEnd(s, k);
		a = s.substring(k, skipTo);
		if(v.indexOf(a) != -1)
		    return true;
		k = skipTo - 1;
	    }
	}
	return false;
    }

    private void VisitGraph(Vector visits, String s)
    {
	Vector v = (Vector)graph.get(s);
	visits.addElement(s);
	if(v != null){
	    Visit(s, visits, v);    
	}
    }

    private void Visit(String s, Vector visits, Vector v)
    {
	String a;
//	System.out.println(s + "\n" + v);
	for(int k = 0; k < v.size(); k++){
	    a = (String)v.elementAt(k);
	    if(visits.indexOf(a) == -1){
		visits.addElement(a);
		VisitGraph(visits, a);
	    }
	}
    }

    private void RemoveNonVisited(Vector visits)
    {
	Vector nonV = new Vector();

	for(answer.FirstVar(); !answer.VarsDone(); answer.NextVar()){
	    if(visits.indexOf(answer.CurrentVar()) == -1){
		nonV.addElement(answer.CurrentVar());
	    }
	}
	RemoveNon(nonV, answer);
    }
}



