package PateLib;

import java.awt.*;
import java.util.*;

/*
---------------------------------------------------------------------------
File	    :		TransformProc.java
Package	    :		PateLib
Author	    :		Ted Hung
Programmed  :		8 Aug 99	
Modified    :
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme, Ted Hung
 * Computer Science Department
 * Duke University
 * August 1999
 * Supported by National Science Foundation DUE-9596002, DUE-9555084,
 *  and DUE-9752583.
 *
 * Copyright (c) 1999
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */


public abstract class TransformProc 
{
    public static final int SHOW_VARIABLE = 1;
    public static final int SHOW_VISUALIZE = 2;
    public static final int SHOW_MODIFY = 4;

    protected Grammar gram;
    protected Frame wnd;
    
    public void setParentFrame(Frame window)
    {
	wnd = window;
    }

    /**
     * Creates a string out of a vector
     */
    protected String makeString(Vector v)
    {
	String s = new String();
	if(v.size() > 0){
	    for (int k = 0; k < v.size() - 1; k++){
		s += v.elementAt(k) + ", ";
	    }
	    //Add last one without ", " at the end
	    s += v.elementAt(v.size() - 1);
	}
	return s;
    }

    /**
     * abstract functions which are called in TransformWindow
     * according to user input
     */
    abstract public void setGrammar(Grammar g);

    /**
     * returns the resulting grammar from this transformation
     */
    abstract public Grammar getAnswer();

    /**
     * checks whether the answer is correct
     */
    abstract public boolean checkAnswer(Grammar curans);

    /**
     * sets the parameters for the GraphFrame window...
     */
    abstract public void modifyGraphFrame(GraphFrame frame);

    /**
     * returns the variables for this transformation
     */
    abstract public Vector getVariables();

    /**
     * returns whether the selected string is correct
     */
    abstract public boolean checkVariables(String check);

    /**
     * returns the description of this transformation
     */
    abstract public String getDescription();

    /**
     * returns a bitfield describing what steps the user
     * should proceed through.
     */
    abstract public int getShowFlags();

    /**
     * returns a vector of Strings describing each section's actions
     */
    abstract public Vector getSectionLabels();

    /**
     * returns a string which describes what type of tranformation this is
     */
    abstract public String getType();

    /**
     * returns the grammar to start the user off with.
     */
    abstract public Grammar getStartGrammar();
}

