package PateLib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
---------------------------------------------------------------------------
File	    :		TransformWindow.java
Package	    :		PateLib  Version 1.1
Author	    :		Ted Hung
Programmed  :		Aug 1999	
Modified    :
---------------------------------------------------------------------------
 *
 * 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme, Ted Hung
 * Computer Science Department
 * Duke University
 * August 1999
 * Supported by National Science Foundation DUE-9596002, DUE-9555084,
 *  and DUE-9752583.
 *
 * Copyright (c) 1999
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

public class TransformWindow extends Frame 
    implements ActionListener, ItemListener
{
    public final static String CONTINUE_STRING = "Continue";
    public final static String SKIP_STRING = "Skip";
    public final static String[] ARESAME_BUTTONS = {CONTINUE_STRING, SKIP_STRING};

    private SaveText Save;
    private Grammar g;
    private String fileName;

    //user interface buttons
    private Button myAddButton;
    private Button myRemoveButton;
    private Button myParseButton;
    private Button myNextButton;
    private Button myShowButton;
    private Button myCheckButton;
    private Button myVisualizeButton;
    private Label  myTitle;
    private Button myShowVarButton;
    private Button myCheckVarButton;

    private Panel myVariablePanel;
    private Label myVariableLabel;
    private boolean myVariableDone;

    private Panel myVisualizePanel;
    private Label myVisualizeLabel;
    private GraphFrame visualFrame;

    private Panel myModifyPanel;
    private Label myModifyLabel;
    private boolean myModifyDone;

    private Panel myNextPanel;
    private Label myNextLabel;

    private TextField myLeftText;
    private TextField myRightText;
    private TextField myVariableText;

    private java.awt.List myPrevRules;
    private java.awt.List myNewRules;
    private Vector myTransProcs;
    private int curproc;

    /**
     *	creates a new transformation window given a grammar, a filename.
     */
    public TransformWindow(Grammar pg, String Filename)
    {
	super();
	setFont(Params.normalFont);
	SetUpMenu();
	g = (Grammar)pg.clone();
	Save = new SaveText(this, Filename);
	setTitle(Params.lambdaName + Save.Filename());
	myTransProcs = new Vector();
	setLayout(new BorderLayout(0,0));

	myVariableDone = true;
	myModifyDone = true;
        visualFrame = new GraphFrame();
        visualFrame.doneVisualizing = true;

	//add the title
	add(myTitle = new Label("Productions", Label.CENTER), "North");
	myTitle.setFont(Params.labelFont);

	//create the variable panel
	myVariablePanel = new Panel(new BorderLayout());
	Panel p = new Panel(new FlowLayout());
	p.add(new Label("Input Variables: "), "North");
	p.add(myVariableText = new TextField(20));
	p.add(myCheckVarButton = new Button("Check"));
	myCheckVarButton.addActionListener(this);
	p.add(myShowVarButton = new Button("Show"));
	myShowVarButton.addActionListener(this);
	myVariablePanel.add(p, "Center");
	myVariablePanel.add(
	    myVariableLabel = new Label("Step 1: Input variables"), "North");
	myVariableLabel.setForeground(Color.white);
	myVariableLabel.setBackground(Color.black);

	myVisualizePanel = new Panel(new BorderLayout());
	p = new Panel(new FlowLayout());
	p.add(myVisualizeButton = new Button("Visualize Productions"));
	myVisualizeButton.addActionListener(this);
	myVisualizePanel.add(p, "Center");
	myVisualizePanel.add(
	    myVisualizeLabel = new Label("Step 2: Visualize the Grammar"), "North");
	myVisualizeLabel.setForeground(Color.white);
	myVisualizeLabel.setBackground(Color.black);

	//separate the individual panels so some can
	//be shown and some can be hidden
	Panel first = new Panel(new BorderLayout(10, 10));
	first.add(myVariablePanel, "North");
	first.add(myVisualizePanel, "Center");

	//add the modify buttons
	myModifyPanel = new Panel(new BorderLayout());
	p = new Panel(new FlowLayout());
	Panel p2 = new Panel(new FlowLayout());
	Panel p3 = new Panel(new BorderLayout(4, 4));

	p2.add(myLeftText = new TextField(10));
	p2.add(new ArrowCanvas(20, 10));
	p2.add(myRightText = new TextField(20));
	p2.add(myAddButton = new Button("Add New Rule"));
	myAddButton.addActionListener(this);
	p3.add(p2, "Center");
	p.add(myRemoveButton = new Button("Remove Selected Rule(s)"));
	myRemoveButton.addActionListener(this);
	myRemoveButton.setEnabled(false);
	p.add(myCheckButton = new Button("Check Answer"));
	myCheckButton.addActionListener(this);
	p.add(myShowButton = new Button("Show Answer"));
	myShowButton.addActionListener(this);
	p3.add(p, "South");
	myModifyPanel.add(p3, "South");

	//add the list boxes
	p = new Panel(new BorderLayout(2,2));
	p2 = new Panel(new BorderLayout());
	p2.add(new Label("Initial Grammar"), "North");
	p2.add(myPrevRules = new java.awt.List(20), "Center");
	setGrammarRules(g, myPrevRules);
	p.add(p2, "East");
	myPrevRules.addItemListener(this);
	myPrevRules.setBackground(new Color(200,200,200));

	p2 = new Panel(new BorderLayout());
	p2.add(new Label("New Grammar"), "North");
	p2.add(myNewRules = new java.awt.List(20), "Center");
	setGrammarRules(g, myNewRules);
	myNewRules.addItemListener(this);
	myNewRules.setForeground(Params.textColor);
	myNewRules.setBackground(Color.white);
	myNewRules.setMultipleMode(true);
	p.add(p2, "Center");
	myModifyPanel.add(p, "Center");
	myModifyPanel.add(
	    myModifyLabel = new Label("Step 3: Modify Grammar"),
	    "North");
	myModifyLabel.setForeground(Color.white);
	myModifyLabel.setBackground(Color.black);

	Panel second = new Panel(new BorderLayout(10, 10));
	second.add(first, "North");
	second.add(myModifyPanel, "Center");
	
	add(second, "Center");

	//add the buttons that are always there
	myNextPanel = new Panel(new BorderLayout(3, 3));
	p = new Panel(new FlowLayout());
	p.add(myParseButton = new Button("Parse"));
	myParseButton.addActionListener(this);
	p.add(myNextButton = new Button("Next Transformation"));
	myNextButton.addActionListener(this);
	myNextPanel.add(p, "Center");
	myNextPanel.add(myNextLabel = new Label("Step 4: Continue"), "North");
	myNextLabel.setForeground(Color.white);
	myNextLabel.setBackground(Color.black);
	
	add(myNextPanel, "South");

	addKeyListener(
	    new KeyAdapter()
	    {
		public void keyPressed(KeyEvent e)
		{
		    TransformWindow.this.keyPressed(e);
		}
	    }
	);

	addWindowListener(
	    new WindowAdapter()
	    {
		public void windowClosing(WindowEvent e)
		{
		    TransformWindow.this.dispose();
		}
	    }
	);
	
	curproc = -1;
	setVisible(true);
	setSize(400,550);
    }

    public void addTransformProc(TransformProc proc)
    {
	myTransProcs.addElement(proc);
    }

    public void nextTransformation()
    {
	if(curproc + 1 >= myTransProcs.size())
	    return;

        //hide the graph frame
        visualFrame.setVisible(false);

	//restore the states
	myAddButton.setEnabled(true);
	myRemoveButton.setEnabled(false);
	myVariableText.setEditable(true);
        myVariableText.setEnabled(true);
	myVariableText.setText("");
	myShowVarButton.setEnabled(true);
	myCheckVarButton.setEnabled(true);
	myShowButton.setEnabled(true);
	myCheckButton.setEnabled(true);
        myLeftText.setText("");
	myRightText.setText("");
        myLeftText.setEditable(true);
        myLeftText.setEnabled(true);
        myRightText.setEditable(true);
	myRightText.setEnabled(true);
	

	++curproc;
	if(myPrevRules.getItemCount() != 0)
	    myPrevRules.removeAll();
	Grammar temp = parseGrammarRules(myNewRules);
	setGrammarRules(temp, myPrevRules);
	TransformProc proc = (TransformProc) myTransProcs.elementAt(curproc);
	proc.setParentFrame(this);
	proc.setGrammar(temp);
	myTitle.setText(Integer.toString(curproc + 1) +
	    ". " + proc.getDescription());
	setTitle(Integer.toString(curproc + 1) +
	    ". " + proc.getDescription());

	int flags = proc.getShowFlags();
	Vector labels = proc.getSectionLabels();
	if((flags & TransformProc.SHOW_VARIABLE) != 0)
	{
	    myVariablePanel.setVisible(true);  
	    myVariableLabel.setText((String) labels.elementAt(0));
	    myVariableDone = false;
	}
	else
	{
	    myVariablePanel.setVisible(false);
	    myVariableDone = true;
	}
	
	if((flags & TransformProc.SHOW_VISUALIZE) != 0)
	{
	    myVisualizePanel.setVisible(true);
	    myVisualizeLabel.setText((String) labels.elementAt(1));
            proc.modifyGraphFrame(visualFrame);
            visualFrame.doneVisualizing = false;
	}
	else
	{
	    myVisualizePanel.setVisible(false);
	    visualFrame.doneVisualizing = true;
	}
	
	if((flags & TransformProc.SHOW_MODIFY) != 0)
	{
	    myModifyPanel.setVisible(true);
	    myModifyLabel.setText((String) labels.elementAt(2));
	    myModifyDone = false;
	}
	else
	{
	    myModifyPanel.setVisible(false);
	    myModifyDone = true;
	}

	myNextLabel.setText((String) labels.elementAt(3));

	if(curproc + 1 >= myTransProcs.size())
	    myNextButton.setVisible(false);
	validate();

	if(proc.getAnswer().equals(temp))
	{
	    TextInDialog tig = new TextInDialog(
		"\nThe Grammar does not contain\nany "+ proc.getType() +
		" productions!\n \nDo you want to continue or\n" + 
		"skip to the next transformation?", 
		ARESAME_BUTTONS, this, "NOTICE:");
	    tig.showResize();
	    String pushedB = tig.getPushedButton();
	    if(pushedB.equals(SKIP_STRING))
	    {
		nextTransformation();
	    }
	}
	proc = (TransformProc) myTransProcs.elementAt(curproc);
        if(myNewRules.getItemCount() != 0)
            myNewRules.removeAll();
        setGrammarRules(proc.getStartGrammar(), myNewRules);
    }

    /**
     *	Sets up the menu for this window
     */
    private void SetUpMenu()
    {
	MenuBar mbar = new MenuBar();
	Menu m;
	m = new Menu("File");
	m.setFont(Params.menuFont);
	MenuItem mi = new MenuItem("Save As...");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mi = new MenuItem("Print");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mi = new MenuItem("-");
	m.add(mi);
	mi = new MenuItem("Close");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mi = new MenuItem("Exit " + Params.Name2);
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mbar.add(m);
	
	m = new Menu("Help");
	m.setFont(Params.menuFont);
	mi = new MenuItem("Help HTML");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mi = new MenuItem("Help Text");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	m.addSeparator();
	mi = new MenuItem("About...");
	mi.setFont(Params.normalFont);
	mi.addActionListener(this);
	m.add(mi);
	mbar.add(m);
	
	setMenuBar(mbar);	
    }

    public void actionPerformed(ActionEvent e)
    {
	TransformProc proc = null;
	if(curproc < myTransProcs.size())
	    proc = (TransformProc) myTransProcs.elementAt(curproc);
	Grammar temp = parseGrammarRules(myNewRules);
	if(e.getSource() instanceof Button)
	{
	    if(e.getSource() == myCheckButton)
	    {
                if(myVariableDone == false || visualFrame.doneVisualizing == false)
		{
		    String str = "You must do the following steps first:\n";
		    if(myVariableDone == false)
			str += myVariableLabel.getText() + "\n";
		    if(visualFrame.doneVisualizing == false)
			str += myVisualizeLabel.getText();
		    OKDialog dg = new OKDialog(str, this, "Show New Grammar");
		    dg.showResize();
		    return;
		}

		if(proc.checkAnswer(temp))
		{
		    OKDialog dg = new OKDialog("All productions are correct!", this, "Check Answer");
		    dg.showResize();
		    myModifyDone = true;
		    myAddButton.setEnabled(false);
		    myRemoveButton.setEnabled(false);
		    myShowButton.setEnabled(false);
		    myCheckButton.setEnabled(false);
		}
		else
		{
		    Grammar ans = proc.getAnswer();
		    boolean incorrectflag = false;
		    for(int listpos = 0; listpos < myNewRules.getItemCount(); ++listpos)
		    {
			if(!ans.IsAProd(myNewRules.getItem(listpos)))
		        {
			    myNewRules.select(listpos);
			    incorrectflag = true;
			}
			else
			{
			    myNewRules.deselect(listpos);
			}
		    }
		    
		    OKDialog dg;
		    if(incorrectflag)
		    {
			dg = new OKDialog(
			    "The highligted productions\n" + 
			    " are incorrect.", 
			    this, "Check Answer");
			dg.showResize();
			return;
		    }

		    int diff = proc.getAnswer().NumProductions() - temp.NumProductions();

		    
		    if(diff > 0)
		    {
			dg = new OKDialog(
			    "Missing " + Integer.toString(diff) +
			    " productions.", 
			    this, "Check Answer");
		    }
		    else if(diff < 0)
		    {
			dg = new OKDialog(
			    Integer.toString(-diff) +
			    " too many productions.", 
			    this, "Check Answer");
		    }
		    else
		    {
			dg = new OKDialog(
			    "The grammar is incorrect.", 
			    this, "Check Answer");
		    }
		    dg.showResize();
		}
		return;
	    }
	    else if(e.getSource() == myShowButton)
	    {
		if(myVariableDone == false || visualFrame.doneVisualizing == false)
		{
		    String str = "You must do the following steps first:\n";
		    if(myVariableDone == false)
			str += myVariableLabel.getText() + "\n";
		    if(visualFrame.doneVisualizing == false)
			str += myVisualizeLabel.getText();
		    OKDialog dg = new OKDialog(str, this, "Show New Grammar");
		    dg.showResize();
		    return;
		}
		if(myNewRules.getItemCount() != 0)
		    myNewRules.removeAll();
    		setGrammarRules(proc.getAnswer(), myNewRules);
		myModifyDone = true;
		myAddButton.setEnabled(false);
		myRemoveButton.setEnabled(false);
		myShowButton.setEnabled(false);
		myCheckButton.setEnabled(false);
                myLeftText.setText("");
	        myRightText.setText("");
                myLeftText.setEditable(false);
                myLeftText.setEnabled(false);
                myRightText.setEditable(false);
	        myRightText.setEnabled(false);
		return;
	    }
	    else if(e.getSource() == myParseButton)
	    {
		Parse();
		return;
	    }
	    else if(e.getSource() == myNextButton)
	    {
		if(myVariableDone == false ||
		   visualFrame.doneVisualizing == false ||
		   myModifyDone == false)
		{
		    String str = "You must do the following steps first:\n";
		    if(myVariableDone == false)
			str += myVariableLabel.getText() + "\n";
		    if(visualFrame.doneVisualizing == false)
			str += myVisualizeLabel.getText() + "\n";
		    if(myModifyDone == false)
			str += myModifyLabel.getText();
		    OKDialog dg = new OKDialog(str, this, "Next Transformation");
		    dg.showResize();
		    return;
		}
		if(!proc.checkAnswer(temp))
		{
		    OKDialog dg = new OKDialog("Can not go to next\ntransformation until all\nproductions are correct!", this, "Warning!");
		    dg.showResize();
		}
		else
		{
		    nextTransformation();
		}
		return;
	    }
	    else if(e.getSource() == myRemoveButton)
	    {
		myRemoveButton.setEnabled(false);
		String selectstrs[] = myNewRules.getSelectedItems();
		for(int x = 0; x < selectstrs.length; ++x)
		{
		    myNewRules.remove(selectstrs[x]);
		}
	    }
	    else if(e.getSource() == myAddButton)
	    {
		if(!myLeftText.getText().equals(""))
		{
		    if(myRightText.getText().equals(""))
		    {
			myNewRules.add(myLeftText.getText() + "->" + 
			    Params.lambdaChar);
		    }
		    else
		    {
			myNewRules.add(myLeftText.getText() + "->" + 
			    myRightText.getText());
		    }
		    myLeftText.setText("");
		    myRightText.setText("");
		}
		else
		{
		    OKDialog dg = new OKDialog("Left hand side cannot be\n a lambda.",
			this, "Error");
		    dg.showResize();
		}
	    }
	    else if(e.getSource() == myVisualizeButton)
	    {
		if(myVariableDone == false)
		{
		    String str = "You must do the following steps first:\n";
		    str += myVariableLabel.getText() + "\n";
		    OKDialog dg = new OKDialog(str, this, "Show New Grammar");
		    dg.showResize();
		    return;
		}
                if(getLocation().x + getSize().width >= Toolkit.getDefaultToolkit().getScreenSize().width - visualFrame.getSize().width &&
                   getLocation().x - visualFrame.getSize().width <= 0)
                   visualFrame.setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - visualFrame.getSize().width)/2,
	                                   (Toolkit.getDefaultToolkit().getScreenSize().height - visualFrame.getSize().height)/2);
                else if(getLocation().x + getSize().width >= Toolkit.getDefaultToolkit().getScreenSize().width - visualFrame.getSize().width)
                    visualFrame.setLocation(getLocation().x - visualFrame.getSize().width,
                                            getLocation().y);
                else
                    visualFrame.setLocation(getLocation().x + getSize().width,
                                            getLocation().y);
		visualFrame.show();
	    }
	    else if(e.getSource() == myCheckVarButton)
	    {
		if(proc.checkVariables(myVariableText.getText()))
		{
		    myVariableDone = true;
		    myVariableText.setEditable(false);
		    myShowVarButton.setEnabled(false);
		    myCheckVarButton.setEnabled(false);
		}
		return;
	    }
	    else if(e.getSource() == myShowVarButton)
	    {
		myVariableText.setText(makeString(proc.getVariables()));
		myVariableDone = true;
		myVariableText.setEditable(false);
		myShowVarButton.setEnabled(false);
		myCheckVarButton.setEnabled(false);
		return;
	    }
	}
	else if(e.getSource() instanceof MenuItem)
	{	
	    String arg = ((MenuItem)e.getSource()).getLabel();
	    if (arg.equals("Save As..."))
	    {
		if(proc.checkAnswer(temp))
		{
		    Save.saveAs(temp);
		    setTitle(Params.lambdaName + Save.Filename());
		}
		else
		{
		    OKDialog dg = new OKDialog("Did not save!!!\nWill not until all productions\nare correct.", this, "Warning!");
		    dg.showResize();
		}
		return;
	    }	
	    else if(arg.equals("Print"))
	    {
		//Print the grammar
		return;
	    }
	    else if (arg.equals("Close"))
	    {
		this.dispose();
		return;
	    }
	    else if (arg.equals("Exit " + Params.Name2))
	    {
		System.exit(3);
		return;
	    }
	    else if(arg.equals("About..."))
	    {
		//Open up the about dialog
		AboutDialog ad = new AboutDialog(this, Params.trans1Pic, "",
				 Params.aboutHeight, Params.aboutWidth);
		ad.show();
		return;
	    }
	    else if(arg.equals("Help HTML"))
	    {
		//HelpDialog hd = new HelpDialog(0, this);
		//hd.show();
		HelpWindow help = HelpWindowHTML.getHelpWindow("PATE",
		    Params._hd_bkcolor);
		help.setVisible(true);
	    }
	    else if(arg.equals("Help Text"))
	    {
		//HelpDialog hd = new HelpDialog(0, this);
		//hd.show();
		HelpWindow help = HelpWindowHTML.getHelpWindow("PATE",
		    Params._hd_bkcolor);
		help.setVisible(true);
	    }
	}
    }

    public void itemStateChanged(ItemEvent e)
    {
	if(e.getSource() == myPrevRules)
	{
	    if(myPrevRules.getSelectedIndex() != -1)
		myPrevRules.deselect(myPrevRules.getSelectedIndex());
	}
	else if(e.getSource() == myNewRules && !myModifyDone)
	{
	    String selectstrs[] = myNewRules.getSelectedItems();
	    myRemoveButton.setEnabled(selectstrs.length > 0);
	}
    }

    /**
     * creates a string out of a vector
     */
    private String makeString(Vector v)
    {
	String s = new String();
	if(v.size() > 0){
	    for (int k = 0; k < v.size() - 1; k++){
		s += v.elementAt(k) + ", ";
	    }
	    //Add last one without ", " at the end
	    s += v.elementAt(v.size() - 1);
	}
	return s;
    }

    /**
     * Handle alt-F4 to exit
     */
    public void keyPressed(KeyEvent e)
    {
	if(KeyEvent.getKeyText(e.getKeyCode()).equals("F4") &&
	   KeyEvent.getKeyModifiersText(e.getModifiers()).equals("Alt"))

	{  // Alt-F4 always exits
	    System.exit(3);
	}
    }

    /**
     * Sets the grammar rules to the specified list
     */
    public void setGrammarRules(Grammar gr, java.awt.List list)
    {
	Grammar gram = (Grammar) gr.clone();
	for(gram.FirstProd(); !gram.ProdsDone(); gram.NextProd())
	{
	    list.add(gram.CurrentProd());
	}
    }

    /**
     * Sets the grammar rules to the specified list
     */
    public Grammar parseGrammarRules(java.awt.List list)
    {
	Grammar result = new Grammar();
	String itemstr[] = list.getItems();
	for(int x = 0; x < list.getItemCount(); ++x)
	{
	    result.addProduction(itemstr[x]);
	}
	return result;
    }

    /**
     * Pops up the Grammar Parser
     */
    public void Parse()
    {
	ParserEnvironment ParserWind = new ParserEnvironment(
	    parseGrammarRules(myNewRules), Save.Filename());
	ParserWind.setSize(360,520);
	ParserWind.show();
    }
}

