/*---------------------------------------------------------------------------
File:         			TreeNode.java
Package:                        PateLib  Version 1.0
Author:				Anna O. Bilska (Anya) V1.0 08/07/96
Modified:                       Ted Hung 8/99
                                
Description of Contents:       
				
--------------------------------------------------------------------------*/

/* 
 * Susan H. Rodger, Anna O. Bilska (Anya), Jason R. Salemme
 * Computer Science Department
 * Duke University
 * August 1996
 * Supported by National Science Foundation DUE-9596002 and DUE-9555084.
 *
 * Copyright (c) 1996
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the author.  The name of the author may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

package PateLib;

import java.util.*;
import java.awt.*;

public class TreeNode
// a node in the path tree
{

   public static final int diameter = 30;

   //holds the data in this node
   public String data;

   //holds the parent node
   public TreeNode parent;

   //holds the list of children of this node
   public Vector children;

   //shows the position of the node on the graph
   public Position coordinate;

   //used to indicate what group of nodes that this
   //node represents.
   public Vector group;
  
   //tells whether this TreeNode is grouped as a result of an
   //unrestricted grammar
   public boolean isGrouped;

   //used to identify what rule produced this node
   private int rulenum = 0;

   public TreeNode(String d, Vector childs, TreeNode par, Position c)
   {
      data = new String(d);
      children = childs;
      parent = par;
      coordinate = c;
      group = null;
      isGrouped = false;
   }

   /**
    * Draws the node or a connector (for data == "" and not the top "")
    * Handles the drawing of nodes for step next
    * at the current position on the canvas
    */ 
   public void drawTreeNode(Graphics g, int curRule)
   {
      if(group != null)
         return;

      FontMetrics fm = g.getFontMetrics();
      int width;
      int height = fm.getAscent();

      String outStr = data;
      if(!data.equals("") ||
         (children.size() > 0 &&
          ((TreeNode)children.elementAt(0)).getRuleNum() > curRule))
      {
         if(children.size() == 0 && !isGrouped)
            g.setColor(Params._scroll_leafnodecolor);
         else 
            g.setColor(Params._scroll_nodecolor);

         g.fillOval(
            coordinate.x - diameter / 2, 
            coordinate.y - diameter / 2,
            diameter, diameter);
         g.setColor(Params._scroll_edgescolor);
         g.drawOval(
            coordinate.x - diameter / 2, 
            coordinate.y - diameter / 2,
            diameter, diameter);

         if(data.equals("") && children.size() > 0 &&
            ((TreeNode)children.elementAt(0)).getRuleNum() > curRule)
         {
            TreeNode curnode = (TreeNode) children.elementAt(0);
            while(curnode != null && curnode.data.equals(""))
            {
               curnode = (TreeNode) curnode.children.elementAt(0);
            }
            outStr = curnode.data;
         }

         width = fm.stringWidth(outStr);
         g.drawString(
            outStr,
            coordinate.x - width/2,
            coordinate.y + height/2);
      }
      else
      {
         g.drawLine(
            coordinate.x,
            coordinate.y - diameter / 2,
            coordinate.x,
            coordinate.y + diameter / 2);
      }
   }

   /**
    * sets the rule that this node corresponds to
    */
   public void setRuleNum(int num)
   {
      rulenum = num;
   }

   /**
    * returns the appropriate level taking into account
    * empty nodes
    */
   public int getRuleNum()
   {
      if(!data.equals("") || parent == null || !parent.data.equals(""))
         return rulenum;

      TreeNode curnode = (TreeNode) children.elementAt(0);
      while(curnode != null && curnode.data.equals(""))
      {
         curnode = (TreeNode) curnode.children.elementAt(0);
      }
      
      if(curnode != null)
         return curnode.getRuleNum();
      return rulenum;
   }

   /**
    * draws the group node for this node if
    * it is a group, otherwise, it draws nothing
    */
   public void drawGroupNode(Graphics g)
   {
      if(group == null)
         return;

      g.fillRect(
         ((TreeNode) group.firstElement()).coordinate.x,
         coordinate.y - diameter / 2 - 3,
         ((TreeNode) group.lastElement()).coordinate.x -
         ((TreeNode) group.firstElement()).coordinate.x,
         diameter + 6);

      g.fillOval(
         ((TreeNode) group.firstElement()).coordinate.x - diameter / 2 - 3, 
         ((TreeNode) group.firstElement()).coordinate.y - diameter / 2 - 3,
         diameter + 6, diameter + 6);

      g.fillOval(
         ((TreeNode) group.lastElement()).coordinate.x - diameter / 2 - 3, 
         ((TreeNode) group.lastElement()).coordinate.y - diameter / 2 - 3,
         diameter + 6, diameter + 6);
   }
}
